"""
MClickBrowserFilePushButton, MClickBrowserFileToolButton
MClickBrowserFolderPushButton, MClickBrowserFolderToolButton
Browser files or folders by selecting.
MDragFileButton, MDragFolderButton
Browser files or folders by dragging.
"""

from pathlib import Path
from typing import TYPE_CHECKING

from qtpy import QtCore, QtGui, QtWidgets

from .mixin import cursor_mixin, property_mixin
from .push_button import MPushButton
from .theme import MTheme
from .tool_button import MToolButton


class MBrowserButton(QtWidgets.QWidget if TYPE_CHECKING else object):
    """
    Browser button mixin class.

    WARNING: This class must be used with multiple inheritance
    alongside a QWidget subclass (MPushButton, MToolButton, etc.)

    Example:
        class MyButton(MPushButton, MBrowserButton):  # Correct
            pass

        class WrongButton(MBrowserButton):  # Wrong - will not work
            pass
    """

    sig_file_changed = QtCore.Signal(str)
    sig_files_changed = QtCore.Signal(list)
    sig_folder_changed = QtCore.Signal(str)
    sig_folders_changed = QtCore.Signal(list)

    def __init__(self, parent: QtWidgets.QWidget | None = None):
        super().__init__()
        self._path: str = ""
        self._multiple: bool = False
        self._filters: list[str] = []

    def get_dayu_filters(self) -> list[str]:
        return self._filters

    def set_dayu_filters(self, value: list[str]) -> None:
        if value != self._filters:
            self._filters = value

    def get_dayu_path(self) -> str:
        return self._path

    def set_dayu_path(self, value: str) -> None:
        if value != self._path:
            self._path = value

    def get_dayu_multiple(self) -> bool:
        return self._multiple

    def set_dayu_multiple(self, value: bool) -> None:
        if value != self._multiple:
            self._multiple = value

    dayu_path = QtCore.Property(str, get_dayu_path, set_dayu_path)
    dayu_multiple = QtCore.Property(bool, get_dayu_multiple, set_dayu_multiple)
    dayu_filters = QtCore.Property(list, get_dayu_filters, set_dayu_filters)

    @QtCore.Slot()
    def slot_browser_file(self) -> None:
        filter_list = (
            f"File({' '.join(['*' + e for e in self.get_dayu_filters()])})"
            if self.get_dayu_filters()
            else "Any File(*)"
        )
        if self.get_dayu_multiple():
            r_files, _ = QtWidgets.QFileDialog.getOpenFileNames(
                self, "Browser File", self.get_dayu_path(), filter_list
            )
            if r_files:
                self.sig_files_changed.emit(r_files)
                self.set_dayu_path(r_files[0])
        else:
            r_file, _ = QtWidgets.QFileDialog.getOpenFileName(
                self, "Browser File", self.get_dayu_path(), filter_list
            )
            if r_file:
                self.sig_file_changed.emit(r_file)
                self.set_dayu_path(r_file)

    @QtCore.Slot()
    def slot_browser_folder(self) -> None:
        r_folder = QtWidgets.QFileDialog.getExistingDirectory(
            self, "Browser Folder", self.get_dayu_path()
        )
        if r_folder:
            self.sig_folder_changed.emit(r_folder)
            self.set_dayu_path(r_folder)

    @QtCore.Slot()
    def slot_save_file(self) -> None:
        filter_list = (
            f"File({' '.join(['*' + e for e in self.get_dayu_filters()])})"
            if self.get_dayu_filters()
            else "Any File(*)"
        )
        r_file, _ = QtWidgets.QFileDialog.getSaveFileName(
            self, "Save File", self.get_dayu_path(), filter_list
        )
        if r_file:
            self.sig_file_changed.emit(r_file)
            self.set_dayu_path(r_file)


class MClickBrowserFilePushButton(MPushButton, MBrowserButton):
    """A Clickable push button to browser files"""

    def __init__(
        self,
        text: str = "Browser",
        multiple: bool = False,
        parent: QtWidgets.QWidget | None = None,
    ):
        super().__init__(text=text, parent=parent)
        self.set_dayu_multiple(multiple)
        self.setToolTip("Click to browser file")
        self.clicked.connect(self.slot_browser_file)


class MClickBrowserFileToolButton(MToolButton, MBrowserButton):
    """A Clickable tool button to browser files"""

    def __init__(
        self,
        multiple: bool = False,
        parent: QtWidgets.QWidget | None = None,
    ):
        super().__init__(parent=parent)
        self.set_dayu_multiple(multiple)
        self.setToolTip("Click to browser file")
        self.svg("cloud_line.svg").icon_only()
        self.clicked.connect(self.slot_browser_file)


class MClickSaveFileToolButton(MToolButton, MBrowserButton):
    """A Clickable tool button to browser files"""

    def __init__(
        self,
        multiple: bool = False,
        parent: QtWidgets.QWidget | None = None,
    ):
        super().__init__(parent=parent)
        self.set_dayu_multiple(multiple)
        self.setToolTip("Click to save file")
        self.svg("save_line.svg").icon_only()
        self.clicked.connect(self.slot_browser_file)


class MDragFileButton(MToolButton, MBrowserButton):
    """A Clickable and draggable tool button to upload files"""

    def __init__(
        self,
        text: str = "",
        multiple: bool = False,
        parent: QtWidgets.QWidget | None = None,
    ):
        super().__init__(parent=parent)
        self.setAcceptDrops(True)
        self.setMouseTracking(True)
        self.set_dayu_multiple(multiple)
        self.setToolTip("Click or drag file here")
        self.svg("cloud_line.svg").text_under_icon().setText(text)

        size = MTheme().drag_size
        self.set_dayu_size(size)
        self.setIconSize(QtCore.QSize(size, size))

        self.setSizePolicy(
            QtWidgets.QSizePolicy.Policy.Expanding,
            QtWidgets.QSizePolicy.Policy.Expanding,
        )
        self.clicked.connect(self.slot_browser_file)

    def dragEnterEvent(self, event: QtGui.QDragEnterEvent) -> None:  # noqa: N802
        """Override dragEnterEvent. Validate dragged files"""
        if event.mimeData().hasFormat("text/uri-list"):
            count = len(event.mimeData().urls())
            if count == 1 or (count > 1 and self.get_dayu_multiple()):
                event.acceptProposedAction()
                return

    def dropEvent(self, event: QtGui.QDropEvent) -> None:  # noqa: N802
        """Override dropEvent to accept the dropped files"""
        file_list = self._get_valid_file_list(event.mimeData().urls())
        if not file_list:
            return
        if self.get_dayu_multiple():
            self.sig_files_changed.emit(file_list)
        else:
            self.sig_file_changed.emit(file_list[0])
        self.set_dayu_path(file_list[0])

    def _get_valid_file_list(self, url_list: list[QtCore.QUrl]) -> list[str]:
        file_list: list[str] = []
        for url in url_list:
            path = Path(url.toLocalFile())
            if path.is_file() and (
                not self.get_dayu_filters() or path.suffix in self.get_dayu_filters()
            ):
                file_list.append(path.as_posix())
        return file_list


class MClickBrowserFolderPushButton(MPushButton, MBrowserButton):
    """A Clickable push button to browser folders"""

    def __init__(
        self,
        text: str = "",
        multiple: bool = False,
        parent: QtWidgets.QWidget | None = None,
    ):
        super().__init__(text=text, parent=parent)
        self.set_dayu_multiple(multiple)
        self.setToolTip("Click to browser folder")
        self.clicked.connect(self.slot_browser_folder)


@property_mixin
class MClickBrowserFolderToolButton(MToolButton, MBrowserButton):
    """A Clickable tool button to browser folders"""

    def __init__(
        self,
        multiple: bool = False,
        parent: QtWidgets.QWidget | None = None,
    ):
        super().__init__(parent=parent)
        self.set_dayu_multiple(multiple)
        self.setToolTip("Click to browser folder")
        self.svg("folder_line.svg").icon_only()
        self.clicked.connect(self.slot_browser_folder)


@property_mixin
@cursor_mixin
class MDragFolderButton(MToolButton, MBrowserButton):
    """A Clickable and draggable tool button to browser folders"""

    def __init__(
        self,
        multiple: bool = False,
        parent: QtWidgets.QWidget | None = None,
    ):
        super().__init__(parent=parent)
        self.setAcceptDrops(True)
        self.setMouseTracking(True)
        self.set_dayu_multiple(multiple)
        self.setToolTip("Click or drag folder here")
        self.svg("folder_line.svg").text_under_icon().setText(
            "Click or drag folder here"
        )

        size = MTheme().drag_size
        self.set_dayu_size(size)
        self.setIconSize(QtCore.QSize(size, size))
        self.setSizePolicy(
            QtWidgets.QSizePolicy.Policy.Expanding,
            QtWidgets.QSizePolicy.Policy.Expanding,
        )

        self.clicked.connect(self.slot_browser_folder)

    def dragEnterEvent(self, event: QtGui.QDragEnterEvent) -> None:  # noqa: N802
        """Override dragEnterEvent. Validate dragged folders"""
        if event.mimeData().hasFormat("text/uri-list"):
            count = len(event.mimeData().urls())
            if count == 1 or (count > 1 and self.get_dayu_multiple()):
                event.acceptProposedAction()
                return

    def dropEvent(self, event: QtGui.QDropEvent) -> None:  # noqa: N802
        """Override dropEvent to accept the dropped folders"""
        folder_list: list[str] = []
        for url in event.mimeData().urls():
            path = Path(url.toLocalFile())
            if path.is_dir():
                folder_list.append(path.as_posix())

        if not folder_list:
            return

        if self.get_dayu_multiple():
            self.sig_folders_changed.emit(folder_list)
        else:
            self.sig_folder_changed.emit(folder_list[0])
        self.set_dayu_path(folder_list[0])



from typing import TYPE_CHECKING, cast

from qtpy import QtCore, QtWidgets

from .theme import MTheme


class MWidget(QtWidgets.QWidget if TYPE_CHECKING else object):
    """
    Base class for all widgets.

    Properties:
        dayu_type: str (default: "")
        dayu_size: int (default: MTheme().default_size)
    """

    def __init__(self, parent: QtWidgets.QWidget | None = None):
        super().__init__()
        self._dayu_type: str = ""
        self._dayu_size: int = MTheme().default_size

    def get_dayu_type(self) -> str:
        return self._dayu_type

    def set_dayu_type(self, value: str):
        if value != self._dayu_type:
            self._dayu_type = value
            self.on_style_changed()

    def get_dayu_size(self) -> int:
        return self._dayu_size

    def set_dayu_size(self, value: int):
        if value != self._dayu_size:
            self._dayu_size = value
            self.on_style_changed()

    def style(self) -> QtWidgets.QStyle: ...

    def on_style_changed(self):
        self.style().polish(cast(QtWidgets.QWidget, self))

    dayu_type = QtCore.Property(str, get_dayu_type, set_dayu_type)
    dayu_size = QtCore.Property(int, get_dayu_size, set_dayu_size)



from qtpy import QtWidgets

import examples.dayu._mock_data as mock
from qtui.dayu.divider import MDivider
from qtui.dayu.field_mixin import MFieldMixin
from qtui.dayu.item_view_full_set import MItemViewFullSet
from qtui.dayu.push_button import MPushButton


class ItemViewFullSetExample(QtWidgets.QWidget, MFieldMixin):
    def __init__(self, parent: QtWidgets.QWidget | None = None):
        super().__init__(parent)
        self._init_ui()

    def _init_ui(self):
        item_view_set_table = MItemViewFullSet()
        item_view_set_table.set_header_list(mock.header_list)

        item_view_set_all = MItemViewFullSet(table_view=True, big_view=True)
        item_view_set_all.set_header_list(mock.header_list)

        refresh_button = MPushButton("Refresh Data")
        refresh_button.clicked.connect(self.slot_update_data)
        main_lay = QtWidgets.QVBoxLayout()
        main_lay.addWidget(MDivider("Only Table View"))
        main_lay.addWidget(refresh_button)
        main_lay.addWidget(item_view_set_table)
        main_lay.addWidget(MDivider("Table View and Big View"))
        main_lay.addWidget(item_view_set_all)
        self.setLayout(main_lay)

        self.view_list = [
            item_view_set_table,
            item_view_set_all,
        ]
        self.slot_update_data()

    def slot_update_data(self):
        for view in self.view_list:
            view.setup_data(mock.data_list)


if __name__ == "__main__":
    from qtui.dayu.qt import application
    from qtui.dayu.theme import MTheme

    with application() as app:
        test = ItemViewFullSetExample()
        MTheme().apply(test)
        test.show()



"""
MDivider
"""

from enum import IntFlag
from typing import ClassVar

from qtpy import QtCore, QtWidgets

from .label import MLabel


class MDivider(QtWidgets.QWidget):
    """
    A divider line separates different content.

    Property:
        dayu_text: str
    """

    _alignment_map: ClassVar[dict[IntFlag, int]] = {
        QtCore.Qt.AlignmentFlag.AlignCenter: 50,
        QtCore.Qt.AlignmentFlag.AlignLeft: 20,
        QtCore.Qt.AlignmentFlag.AlignRight: 80,
    }

    def __init__(
        self,
        text: str = "",
        orientation: QtCore.Qt.Orientation = QtCore.Qt.Orientation.Horizontal,
        alignment: QtCore.Qt.AlignmentFlag = QtCore.Qt.AlignmentFlag.AlignCenter,
        parent: QtWidgets.QWidget | None = None,
    ):
        super().__init__(parent)
        self._orient = orientation
        self._text_label = MLabel().secondary()
        self._left_frame = QtWidgets.QFrame()
        self._right_frame = QtWidgets.QFrame()
        self._main_lay = QtWidgets.QHBoxLayout()
        self._main_lay.setContentsMargins(0, 0, 0, 0)
        self._main_lay.setSpacing(0)
        self._main_lay.addWidget(self._left_frame)
        self._main_lay.addWidget(self._text_label)
        self._main_lay.addWidget(self._right_frame)
        self.setLayout(self._main_lay)

        if orientation == QtCore.Qt.Orientation.Horizontal:
            self._left_frame.setFrameShape(QtWidgets.QFrame.Shape.HLine)
            self._left_frame.setFrameShadow(QtWidgets.QFrame.Shadow.Sunken)
            self._right_frame.setFrameShape(QtWidgets.QFrame.Shape.HLine)
            self._right_frame.setFrameShadow(QtWidgets.QFrame.Shadow.Sunken)
        else:
            self._text_label.setVisible(False)
            self._right_frame.setVisible(False)
            self._left_frame.setFrameShape(QtWidgets.QFrame.Shape.VLine)
            self._left_frame.setFrameShadow(QtWidgets.QFrame.Shadow.Plain)
            self.setFixedWidth(2)
        self._main_lay.setStretchFactor(
            self._left_frame, self._alignment_map.get(alignment, 50)
        )
        self._main_lay.setStretchFactor(
            self._right_frame, 100 - self._alignment_map.get(alignment, 50)
        )
        self._text = None
        self.set_dayu_text(text)

    def set_dayu_text(self, value: str):
        """
        Set the divider's text.
        When text is empty, hide the text_label and right_frame
        to ensure the divider not has a gap.

        :param value: str
        :return: None
        """
        self._text = value
        self._text_label.setText(value)
        if self._orient == QtCore.Qt.Orientation.Horizontal:
            self._text_label.setVisible(bool(value))
            self._right_frame.setVisible(bool(value))

    def get_dayu_text(self):
        """
        Get current text
        :return: str
        """
        return self._text

    dayu_text = QtCore.Property(str, get_dayu_text, set_dayu_text)

    @classmethod
    def left(cls, text: str = ""):
        """Create a horizontal divider with text at left."""
        return cls(text, alignment=QtCore.Qt.AlignmentFlag.AlignLeft)

    @classmethod
    def right(cls, text: str = ""):
        """Create a horizontal divider with text at right."""
        return cls(text, alignment=QtCore.Qt.AlignmentFlag.AlignRight)

    @classmethod
    def center(cls, text: str = ""):
        """Create a horizontal divider with text at center."""
        return cls(text, alignment=QtCore.Qt.AlignmentFlag.AlignCenter)

    @classmethod
    def vertical(cls):
        """Create a vertical divider"""
        return cls(orientation=QtCore.Qt.Orientation.Vertical)



import functools

from qtpy import QtCore, QtWidgets

from qtui.dayu.button_group import MPushButtonGroup
from qtui.dayu.divider import MDivider
from qtui.dayu.field_mixin import MFieldMixin
from qtui.dayu.label import MLabel
from qtui.dayu.progress_circle import MProgressCircle
from qtui.dayu.push_button import MPushButton
from qtui.dayu.qt import get_scale_factor
from qtui.dayu.theme import MTheme
from qtui.dayu.types import MButtonGroupData


class ProgressCircleExample(QtWidgets.QWidget, MFieldMixin):
    def __init__(self, parent: QtWidgets.QWidget | None = None):
        super().__init__(parent)
        self.setWindowTitle("Examples for MProgressCircle")
        self._init_ui()

    def _init_ui(self):
        main_lay = QtWidgets.QVBoxLayout()
        self.setLayout(main_lay)
        main_lay.addWidget(MDivider("circle"))
        lay1 = QtWidgets.QHBoxLayout()
        circle_1 = MProgressCircle(parent=self)
        circle_1.setFormat("%p Days")
        circle_1.setValue(80)
        circle_2 = MProgressCircle(parent=self)
        circle_2.set_dayu_color(MTheme().success_color)
        circle_2.setValue(100)
        circle_3 = MProgressCircle(parent=self)
        circle_3.set_dayu_color(MTheme().error_color)
        circle_3.setValue(40)

        dashboard_1 = MProgressCircle.dashboard(parent=self)
        dashboard_1.setFormat("%p Days")
        dashboard_1.setValue(80)
        dashboard_2 = MProgressCircle.dashboard(parent=self)
        dashboard_2.set_dayu_color(MTheme().success_color)
        dashboard_2.setValue(100)
        dashboard_3 = MProgressCircle.dashboard(parent=self)
        dashboard_3.set_dayu_color(MTheme().error_color)
        dashboard_3.setValue(40)

        lay1.addWidget(circle_1)
        lay1.addWidget(circle_2)
        lay1.addWidget(circle_3)

        dashboard_lay = QtWidgets.QHBoxLayout()
        dashboard_lay.addWidget(dashboard_1)
        dashboard_lay.addWidget(dashboard_2)
        dashboard_lay.addWidget(dashboard_3)
        main_lay.addLayout(lay1)
        main_lay.addWidget(MDivider("dashboard"))
        main_lay.addLayout(dashboard_lay)
        main_lay.addWidget(MDivider("different radius"))

        scale_x, _ = get_scale_factor()
        circle_4 = MProgressCircle(parent=self)
        circle_4.set_dayu_width(int(100 * scale_x))
        circle_4.setValue(40)
        circle_5 = MProgressCircle(parent=self)
        circle_5.setValue(40)
        circle_6 = MProgressCircle(parent=self)
        circle_6.set_dayu_width(int(160 * scale_x))
        circle_6.setValue(40)
        lay2 = QtWidgets.QHBoxLayout()
        lay2.addWidget(circle_4)
        lay2.addWidget(circle_5)
        lay2.addWidget(circle_6)

        main_lay.addLayout(lay2)
        main_lay.addWidget(MDivider("data bind"))

        self.register_field("percent", 0)
        self.register_field("color", self.get_color)
        self.register_field("format", self.get_format)
        circle = MProgressCircle(parent=self)

        self.bind("percent", circle, "value")
        self.bind("color", circle, "dayu_color")
        self.bind("format", circle, "format")
        lay3 = QtWidgets.QHBoxLayout()
        button_grp = MPushButtonGroup()
        button_grp.set_dayu_type(MPushButton.ButtonType.Default)
        button_grp.set_button_list(
            [
                MButtonGroupData(
                    text="+",
                    clicked=functools.partial(self.slot_change_percent, 10),
                ),
                MButtonGroupData(
                    text="-",
                    clicked=functools.partial(self.slot_change_percent, -10),
                ),
            ]
        )
        lay3.addWidget(circle)
        lay3.addWidget(button_grp)
        lay3.addStretch()
        main_lay.addLayout(lay3)

        custom_widget = QtWidgets.QWidget()
        custom_layout = QtWidgets.QVBoxLayout()
        custom_layout.setContentsMargins(10, 10, 10, 10)
        custom_layout.addStretch()
        custom_widget.setLayout(custom_layout)
        lab1 = MLabel(text="42,001,776").h3()
        lab2 = MLabel(text="Consumer population size").secondary()
        lab3 = MLabel(text="Total population 75%").secondary()
        lab1.setAlignment(QtCore.Qt.AlignmentFlag.AlignCenter)
        lab2.setAlignment(QtCore.Qt.AlignmentFlag.AlignCenter)
        lab3.setAlignment(QtCore.Qt.AlignmentFlag.AlignCenter)
        custom_layout.addWidget(lab1)
        custom_layout.addWidget(lab2)
        custom_layout.addWidget(MDivider())
        custom_layout.addWidget(lab3)
        custom_layout.addStretch()
        custom_circle = MProgressCircle()
        custom_circle.set_dayu_width(int(250 * scale_x))
        custom_circle.setValue(75)
        custom_circle.set_widget(custom_widget)

        main_lay.addWidget(MDivider("custom circle"))
        main_lay.addWidget(custom_circle)
        main_lay.addStretch()

    def get_color(self):
        p = self.field("percent")
        if p < 30:
            return MTheme().error_color
        if p < 60:
            return MTheme().warning_color
        if p < 100:
            return MTheme().primary_color
        return MTheme().success_color

    def get_format(self):
        p = self.field("percent")
        if p < 30:
            return ">_<"
        if p < 60:
            return "0_0"
        if p < 100:
            return "^_^"
        return "^o^"

    def slot_change_percent(self, value: int):
        self.set_field("percent", max(0, min(self.field("percent") + value, 100)))


if __name__ == "__main__":
    from qtui.dayu.qt import application
    from qtui.dayu.theme import MTheme

    with application() as app:
        test = ProgressCircleExample()
        MTheme().apply(test)
        test.show()



"""
MLoading
"""

from qtpy import QtCore, QtGui, QtWidgets

from .qt import MPixmap
from .theme import MTheme


class MLoading(QtWidgets.QWidget):
    """
    Show a loading animation image.
    """

    def __init__(
        self,
        size: int | None = None,
        color: str | None = None,
        parent: QtWidgets.QWidget | None = None,
    ):
        super().__init__(parent)
        size = size or MTheme().default_size
        self.setFixedSize(QtCore.QSize(size, size))
        self.pix = MPixmap(
            "loading.svg", color or MTheme().primary_color
        ).scaledToWidth(size, QtCore.Qt.TransformationMode.SmoothTransformation)
        self._rotation: int = 0
        self._loading_ani = QtCore.QPropertyAnimation()
        self._loading_ani.setTargetObject(self)
        self._loading_ani.setDuration(1000)
        self._loading_ani.setPropertyName(b"rotation")
        self._loading_ani.setStartValue(0)
        self._loading_ani.setEndValue(360)
        self._loading_ani.setLoopCount(-1)
        self._loading_ani.start()

    def _set_rotation(self, value: int):
        self._rotation = value
        self.update()

    def _get_rotation(self) -> int:
        return self._rotation

    rotation = QtCore.Property(int, _get_rotation, _set_rotation)

    def paintEvent(self, event: QtGui.QPaintEvent):  # noqa: N802
        """override the paint event to paint the 1/4 circle image."""
        painter = QtGui.QPainter(self)
        painter.setRenderHint(QtGui.QPainter.RenderHint.SmoothPixmapTransform)
        painter.translate(int(self.pix.width() / 2), int(self.pix.height() / 2))
        painter.rotate(self._rotation)
        painter.drawPixmap(
            int(-self.pix.width() / 2),
            int(-self.pix.height() / 2),
            self.pix.width(),
            self.pix.height(),
            self.pix,
        )
        painter.end()
        return super().paintEvent(event)

    @classmethod
    def huge(cls, color: str | None = None):
        """Create a MLoading with huge size"""
        return cls(MTheme().huge, color)

    @classmethod
    def large(cls, color: str | None = None):
        """Create a MLoading with large size"""
        return cls(MTheme().large, color)

    @classmethod
    def medium(cls, color: str | None = None):
        """Create a MLoading with medium size"""
        return cls(MTheme().medium, color)

    @classmethod
    def small(cls, color: str | None = None):
        """Create a MLoading with small size"""
        return cls(MTheme().small, color)

    @classmethod
    def tiny(cls, color: str | None = None):
        """Create a MLoading with tiny size"""
        return cls(MTheme().tiny, color)


class MLoadingWrapper(QtWidgets.QWidget):
    """
    A wrapper widget to show the loading widget or hide.
    Property:
        dayu_loading: bool. current loading state.
    """

    def __init__(
        self,
        widget: QtWidgets.QWidget,
        loading: bool = True,
        parent: QtWidgets.QWidget | None = None,
    ):
        super().__init__(parent)
        self._widget = widget
        self._mask_widget = QtWidgets.QFrame()
        self._mask_widget.setObjectName("mask")
        self._mask_widget.setSizePolicy(
            QtWidgets.QSizePolicy.Policy.Expanding,
            QtWidgets.QSizePolicy.Policy.Expanding,
        )
        self._loading_widget = MLoading()
        self._loading_widget.setSizePolicy(
            QtWidgets.QSizePolicy.Policy.Expanding,
            QtWidgets.QSizePolicy.Policy.Expanding,
        )

        self._main_lay = QtWidgets.QGridLayout()
        self._main_lay.setContentsMargins(0, 0, 0, 0)
        self._main_lay.addWidget(widget, 0, 0)
        self._main_lay.addWidget(self._mask_widget, 0, 0)
        self._main_lay.addWidget(
            self._loading_widget, 0, 0, QtCore.Qt.AlignmentFlag.AlignCenter
        )
        self.setLayout(self._main_lay)
        self._loading: bool = False
        self.set_dayu_loading(loading)

    def _set_loading(self):
        self._loading_widget.setVisible(self._loading)
        self._mask_widget.setVisible(self._loading)

    def set_dayu_loading(self, loading: bool):
        """
        Set current state to loading or not
        :param loading: bool
        :return: None
        """
        self._loading = loading
        self._set_loading()

    def get_dayu_loading(self) -> bool:
        """
        Get current loading widget is loading or not.
        :return: bool
        """
        return self._loading

    dayu_loading = QtCore.Property(bool, get_dayu_loading, set_dayu_loading)



"""
mixin decorators to add Qt class feature.
"""

from typing import Any, TypeVar, cast

from qtpy import QtCore, QtGui, QtWidgets

T = TypeVar(
    "T",
    bound=QtWidgets.QWidget,
)


def property_mixin(
    cls: type[T],
) -> type[T]:
    """Run function after dynamic property value changed"""

    def _new_event(self: T, event: QtCore.QEvent) -> bool:
        if event.type() == QtCore.QEvent.Type.DynamicPropertyChange:
            event = cast(QtCore.QDynamicPropertyChangeEvent, event)
            prp = cast(str, event.propertyName().data().decode())  # pyright: ignore[reportUnknownMemberType, reportAttributeAccessIssue]
            if hasattr(self, f"_set_{prp}"):
                callback = getattr(self, f"_set_{prp}")
                callback(self.property(str(prp)))
        return super(cls, self).event(event)

    cls.event = _new_event
    return cls


def cursor_mixin(cls: type[T]) -> type[T]:
    """
    Change Widget cursor:
    when user mouse in: Qt.PointingHandCursor;
    when widget is disabled and mouse in: Qt.ForbiddenCursor
    """

    old_enter_event = cls.enterEvent
    old_leave_event = cls.leaveEvent
    old_hide_event = cls.hideEvent
    old_focus_out_event = cls.focusOutEvent

    def _revert_cursor(self: T) -> None:
        if self.__dict__.get("__dayu_enter", False):
            while self.__dict__.get("__dayu_enter_count", 0) > 0:
                QtWidgets.QApplication.restoreOverrideCursor()
                self.__dict__.update(
                    {
                        "__dayu_enter_count": self.__dict__.get("__dayu_enter_count", 0)
                        - 1
                    }
                )
            self.__dict__.update({"__dayu_enter": False})

    def _new_enter_event(self: T, *args: Any, **kwargs: Any) -> None:
        self.__dict__.update({"__dayu_enter": True})
        self.__dict__.update(
            {"__dayu_enter_count": self.__dict__.get("__dayu_enter_count", 0) + 1}
        )
        QtWidgets.QApplication.setOverrideCursor(
            QtCore.Qt.CursorShape.PointingHandCursor
            if self.isEnabled()
            else QtCore.Qt.CursorShape.ForbiddenCursor
        )
        return old_enter_event(self, *args, **kwargs)

    def _new_leave_event(self: T, *args: Any, **kwargs: Any) -> None:
        _revert_cursor(self)
        return old_leave_event(self, *args, **kwargs)

    def _new_hide_event(self: T, *args: Any, **kwargs: Any) -> None:
        _revert_cursor(self)
        return old_hide_event(self, *args, **kwargs)

    def _new_focus_out_event(self: T, *args: Any, **kwargs: Any) -> None:
        _revert_cursor(self)
        return old_focus_out_event(self, *args, **kwargs)

    cls.enterEvent = _new_enter_event
    cls.leaveEvent = _new_leave_event
    cls.hideEvent = _new_hide_event
    cls.focusOutEvent = _new_focus_out_event
    return cls


def focus_shadow_mixin(cls: type[T]) -> type[T]:
    """
    Add shadow effect for decorated class when widget focused
    When focus in target widget, enable shadow effect.
    When focus out target widget, disable shadow effect.
    """
    old_focus_in_event = cls.focusInEvent
    old_focus_out_event = cls.focusOutEvent

    def _new_focus_in_event(self: T, *args: Any, **kwargs: Any) -> None:
        old_focus_in_event(self, *args, **kwargs)
        graphics_effects = self.graphicsEffect()
        if not graphics_effects:
            from .theme import MTheme

            shadow_effect = QtWidgets.QGraphicsDropShadowEffect(self)
            dayu_type = self.property("dayu_type")
            color = vars(MTheme()).get(f"{dayu_type or 'primary'}_color")
            shadow_effect.setColor(QtGui.QColor(color))
            shadow_effect.setOffset(0, 0)
            shadow_effect.setBlurRadius(5)
            shadow_effect.setEnabled(False)
            self.setGraphicsEffect(shadow_effect)
        else:
            if self.isEnabled():
                graphics_effects.setEnabled(True)

    def _new_focus_out_event(self: T, *args: Any, **kwargs: Any) -> None:
        old_focus_out_event(self, *args, **kwargs)
        graphics_effects = self.graphicsEffect()
        if graphics_effects:
            graphics_effects.setEnabled(False)

    cls.focusInEvent = _new_focus_in_event
    cls.focusOutEvent = _new_focus_out_event
    return cls


def hover_shadow_mixin(cls: type[T]) -> type[T]:
    """
    Add shadow effect for decorated class when widget hovered
    When mouse enter target widget, enable shadow effect.
    When mouse leave target widget, disable shadow effect.
    """
    old_enter_event = cls.enterEvent
    old_leave_event = cls.leaveEvent

    def _new_enter_event(self: T, *args: Any, **kwargs: Any) -> None:
        old_enter_event(self, *args, **kwargs)
        graphics_effects = self.graphicsEffect()
        if not graphics_effects:
            from .theme import MTheme

            shadow_effect = QtWidgets.QGraphicsDropShadowEffect(self)
            dayu_type = self.property("type")
            color = vars(MTheme()).get(f"{dayu_type or 'primary'}_color")
            shadow_effect.setColor(QtGui.QColor(color))
            shadow_effect.setOffset(0, 0)
            shadow_effect.setBlurRadius(5)
            shadow_effect.setEnabled(False)
            self.setGraphicsEffect(shadow_effect)
        else:
            if self.isEnabled():
                graphics_effects.setEnabled(True)

    def _new_leave_event(self: T, *args: Any, **kwargs: Any) -> None:
        old_leave_event(self, *args, **kwargs)
        graphics_effects = self.graphicsEffect()
        if graphics_effects:
            graphics_effects.setEnabled(False)

    cls.enterEvent = _new_enter_event
    cls.leaveEvent = _new_leave_event
    return cls


def _stackable(widget: Any) -> bool:
    """Used for stacked_animation_mixin to only add mixin for widget who can stacked."""
    # We use widget() to get currentWidget, use currentChanged to play the animation.
    # For now just QTabWidget and QStackedWidget can use this decorator.
    return (
        issubclass(widget, QtWidgets.QWidget)
        and hasattr(widget, "widget")
        and hasattr(widget, "currentChanged")
    )


def stacked_animation_mixin(cls: type[T]) -> type[T]:
    """
    Decorator for stacked widget.
    When Stacked widget currentChanged, show opacity and
    position animation for current widget.
    """
    if not _stackable(cls):  # If widget can't stack, return the original widget class
        return cls
    old_init = cls.__init__

    def _new_init(self: T, *args: Any, **kwargs: Any):
        old_init(self, *args, **kwargs)
        self._previous_index = 0  # pyright: ignore[reportAttributeAccessIssue]
        self._to_show_pos_ani = QtCore.QPropertyAnimation()  # pyright: ignore[reportAttributeAccessIssue]
        self._to_show_pos_ani.setDuration(400)  # pyright: ignore[reportAttributeAccessIssue]
        self._to_show_pos_ani.setPropertyName(b"pos")  # pyright: ignore[reportAttributeAccessIssue]
        self._to_show_pos_ani.setEndValue(QtCore.QPoint(0, 0))  # pyright: ignore[reportAttributeAccessIssue]
        self._to_show_pos_ani.setEasingCurve(QtCore.QEasingCurve.Type.OutCubic)  # pyright: ignore[reportAttributeAccessIssue]

        self._to_hide_pos_ani = QtCore.QPropertyAnimation()  # pyright: ignore[reportAttributeAccessIssue]
        self._to_hide_pos_ani.setDuration(400)  # pyright: ignore[reportAttributeAccessIssue]
        self._to_hide_pos_ani.setPropertyName(b"pos")  # pyright: ignore[reportAttributeAccessIssue]
        self._to_hide_pos_ani.setEndValue(QtCore.QPoint(0, 0))  # pyright: ignore[reportAttributeAccessIssue]
        self._to_hide_pos_ani.setEasingCurve(QtCore.QEasingCurve.Type.OutCubic)  # pyright: ignore[reportAttributeAccessIssue]

        self._opacity_eff = QtWidgets.QGraphicsOpacityEffect()  # pyright: ignore[reportAttributeAccessIssue]
        self._opacity_ani = QtCore.QPropertyAnimation()  # pyright: ignore[reportAttributeAccessIssue]
        self._opacity_ani.setDuration(400)  # pyright: ignore[reportAttributeAccessIssue]
        self._opacity_ani.setEasingCurve(QtCore.QEasingCurve.Type.InCubic)  # pyright: ignore[reportAttributeAccessIssue]
        self._opacity_ani.setPropertyName(b"opacity")  # pyright: ignore[reportAttributeAccessIssue]
        self._opacity_ani.setStartValue(0.0)  # pyright: ignore[reportAttributeAccessIssue]
        self._opacity_ani.setEndValue(1.0)  # pyright: ignore[reportAttributeAccessIssue]
        self._opacity_ani.setTargetObject(self._opacity_eff)  # pyright: ignore[reportAttributeAccessIssue]
        self._opacity_ani.finished.connect(self._disable_opacity)  # pyright: ignore[reportAttributeAccessIssue, reportUnknownMemberType, reportUnknownArgumentType]
        self.currentChanged.connect(self._play_anim)  # pyright: ignore[reportUnknownMemberType, reportAttributeAccessIssue]

    def _play_anim(self: T, index: int):
        current_widget = self.widget(index)  # pyright: ignore[reportUnknownMemberType, reportAttributeAccessIssue, reportUnknownVariableType]
        if self._previous_index < index:  # pyright: ignore[reportUnknownMemberType, reportAttributeAccessIssue]
            self._to_show_pos_ani.setStartValue(QtCore.QPoint(self.width(), 0))  # pyright: ignore[reportUnknownMemberType, reportAttributeAccessIssue]
            self._to_show_pos_ani.setTargetObject(current_widget)  # pyright: ignore[reportUnknownMemberType, reportAttributeAccessIssue]
            self._to_show_pos_ani.start()  # pyright: ignore[reportUnknownMemberType, reportAttributeAccessIssue]
        else:
            self._to_hide_pos_ani.setStartValue(QtCore.QPoint(-self.width(), 0))  # pyright: ignore[reportUnknownMemberType, reportAttributeAccessIssue]
            self._to_hide_pos_ani.setTargetObject(current_widget)  # pyright: ignore[reportUnknownMemberType, reportAttributeAccessIssue]
            self._to_hide_pos_ani.start()  # pyright: ignore[reportUnknownMemberType, reportAttributeAccessIssue]
        current_widget.setGraphicsEffect(self._opacity_eff)  # pyright: ignore[reportUnknownMemberType, reportAttributeAccessIssue]
        current_widget.graphicsEffect().setEnabled(True)  # pyright: ignore[reportUnknownMemberType]
        self._opacity_ani.start()  # pyright: ignore[reportUnknownMemberType, reportAttributeAccessIssue]
        self._previous_index = index  # pyright: ignore[reportAttributeAccessIssue]

    def _disable_opacity(self: T):
        self.currentWidget().graphicsEffect().setEnabled(False)  # pyright: ignore[reportUnknownMemberType, reportAttributeAccessIssue]

    cls.__init__ = _new_init
    cls._play_anim = _play_anim  # pyright: ignore[reportAttributeAccessIssue]
    cls._disable_opacity = _disable_opacity  # pyright: ignore[reportAttributeAccessIssue]
    return cls



import functools

from qtpy import QtCore, QtGui, QtWidgets

from . import utils
from .menu import MMenu


class MHeaderView(QtWidgets.QHeaderView):
    def __init__(
        self,
        orientation: QtCore.Qt.Orientation,
        parent: QtWidgets.QWidget | None = None,
    ):
        super().__init__(orientation, parent)
        self.setMovable(True)
        self.setClickable(True)
        self.setSortIndicatorShown(True)
        self.setContextMenuPolicy(QtCore.Qt.ContextMenuPolicy.CustomContextMenu)
        self.customContextMenuRequested.connect(self._slot_context_menu)
        self.setDefaultAlignment(
            QtCore.Qt.AlignmentFlag.AlignLeft | QtCore.Qt.AlignmentFlag.AlignVCenter
        )
        self.setProperty(
            "orientation",
            "horizontal"
            if orientation == QtCore.Qt.Orientation.Horizontal
            else "vertical",
        )

    @QtCore.Slot(QtCore.QPoint)
    def _slot_context_menu(self, point: QtCore.QPoint):
        context_menu = MMenu(parent=self)
        logical_column = self.logicalIndexAt(point)
        model = utils.real_model(self.model())
        if not model:
            return
        if logical_column >= 0 and model.header_list[logical_column].get(  # pyright: ignore[reportUnknownMemberType, reportAttributeAccessIssue]
            "checkable", False
        ):
            action_select_all = context_menu.addAction(self.tr("Select All"))
            action_select_none = context_menu.addAction(self.tr("Select None"))
            action_select_invert = context_menu.addAction(self.tr("Select Invert"))
            action_select_all.triggered.connect(
                functools.partial(
                    self._slot_set_select, logical_column, QtCore.Qt.CheckState.Checked
                )
            )
            action_select_none.triggered.connect(
                functools.partial(
                    self._slot_set_select,
                    logical_column,
                    QtCore.Qt.CheckState.Unchecked,
                )
            )
            action_select_invert.triggered.connect(
                functools.partial(self._slot_set_select, logical_column, None)
            )
            context_menu.addSeparator()

        fit_action = context_menu.addAction(self.tr("Fit Size"))
        fit_action.triggered.connect(
            functools.partial(self._slot_set_resize_mode, True)
        )
        context_menu.addSeparator()
        for column in range(self.count()):
            action = context_menu.addAction(
                str(
                    model.headerData(
                        column,
                        QtCore.Qt.Orientation.Horizontal,
                        QtCore.Qt.ItemDataRole.DisplayRole,
                    )
                )
            )
            action.setCheckable(True)
            action.setChecked(not self.isSectionHidden(column))
            action.toggled.connect(
                functools.partial(self._slot_set_section_visible, column)
            )
        context_menu.exec(QtGui.QCursor.pos() + QtCore.QPoint(10, 10))

    @QtCore.Slot(int, int)
    def _slot_set_select(self, column: int, state: QtCore.Qt.CheckState | None):
        current_model = self.model()
        source_model = utils.real_model(current_model)
        if not source_model:
            return
        source_model.beginResetModel()
        attr = f"{source_model.header_list[column].get('key')}_checked"  # pyright: ignore[reportUnknownMemberType, reportAttributeAccessIssue]
        for row in range(current_model.rowCount()):
            real_index = utils.real_index(current_model.index(row, column))
            data_obj = real_index.internalPointer()
            if state is None:
                old_state = utils.get_obj_value(data_obj, attr)
                utils.set_obj_value(
                    data_obj,
                    attr,
                    QtCore.Qt.CheckState.Unchecked
                    if old_state == QtCore.Qt.CheckState.Checked
                    else QtCore.Qt.CheckState.Checked,
                )
            else:
                utils.set_obj_value(data_obj, attr, state)
        source_model.endResetModel()
        source_model.dataChanged.emit(None, None)

    @QtCore.Slot(QtCore.QModelIndex, int)
    def _slot_set_section_visible(self, index: int, flag: bool):
        self.setSectionHidden(index, not flag)

    @QtCore.Slot(bool)
    def _slot_set_resize_mode(self, flag: bool):
        if flag:
            self.resizeSections(QtWidgets.QHeaderView.ResizeMode.ResizeToContents)
        else:
            self.resizeSections(QtWidgets.QHeaderView.ResizeMode.Interactive)

    def setClickable(self, flag: bool):  # noqa: N802
        QtWidgets.QHeaderView.setSectionsClickable(self, flag)

    def setMovable(self, flag: bool):  # noqa: N802
        QtWidgets.QHeaderView.setSectionsMovable(self, flag)

    def resizeMode(self, index: int):  # noqa: N802
        QtWidgets.QHeaderView.sectionResizeMode(self, index)

    def setResizeMode(self, mode: QtWidgets.QHeaderView.ResizeMode):  # noqa: N802
        QtWidgets.QHeaderView.setSectionResizeMode(self, mode)



from .alert import MAlert
from .avatar import MAvatar
from .badge import MBadge
from .breadcrumb import MBreadcrumb
from .browser import (
    MBrowserButton,
    MClickBrowserFilePushButton,
    MClickBrowserFileToolButton,
    MClickBrowserFolderPushButton,
    MClickBrowserFolderToolButton,
    MClickSaveFileToolButton,
    MDragFileButton,
    MDragFolderButton,
)
from .button_group import (
    MCheckBoxGroup,
    MPushButtonGroup,
    MRadioButtonGroup,
    MToolButtonGroup,
)
from .card import MCard, MMetaCard
from .carousel import MCarousel
from .check_box import MCheckBox
from .collapse import MCollapse
from .color_palette import MColorChart, MColorPaletteDialog
from .combo_box import MComboBox
from .completer import MCompleter
from .divider import MDivider
from .dock_widget import MDockWidget
from .drawer import MDrawer
from .field_mixin import MFieldMixin
from .flow_layout import MFlowLayout
from .form import MForm
from .header_view import MHeaderView
from .item_model import MSortFilterModel, MTableModel
from .item_view import (
    MAbstractView,
    MBigView,
    MListView,
    MOptionDelegate,
    MTableView,
    MTreeView,
)
from .item_view_full_set import MItemViewFullSet
from .item_view_set import MItemViewSet
from .label import MLabel
from .line_edit import MLineEdit
from .line_tab_widget import MLineTabWidget, MUnderlineButton, MUnderlineButtonGroup
from .loading import MLoading, MLoadingWrapper
from .menu import MMenu
from .menu_tab_widget import MBlockButton, MBlockButtonGroup, MMenuTabWidget
from .message import MMessage
from .page import MPage
from .popup import MPopup
from .progress_bar import MProgressBar
from .progress_circle import MProgressCircle
from .push_button import MPushButton
from .radio_button import MRadioButton
from .slider import MSlider
from .spin_box import MDateEdit, MDateTimeEdit, MDoubleSpinBox, MSpinBox, MTimeEdit
from .splitter import MSplitter
from .stacked_widget import MStackedWidget
from .switch import MSwitch
from .tab_widget import MTabBar, MTabWidget
from .text_edit import MTextEdit
from .theme import MTheme
from .toast import MToast
from .tool_button import MToolButton
from .widget import MWidget

__all__ = [
    "MAlert",
    "MAvatar",
    "MBadge",
    "MBreadcrumb",
    "MBrowserButton",
    "MClickBrowserFilePushButton",
    "MClickBrowserFileToolButton",
    "MClickBrowserFolderPushButton",
    "MClickBrowserFolderToolButton",
    "MClickSaveFileToolButton",
    "MDragFileButton",
    "MDragFolderButton",
    "MCheckBoxGroup",
    "MPushButtonGroup",
    "MRadioButtonGroup",
    "MToolButtonGroup",
    "MCard",
    "MMetaCard",
    "MCarousel",
    "MCheckBox",
    "MCollapse",
    "MColorChart",
    "MColorPaletteDialog",
    "MComboBox",
    "MCompleter",
    "MDivider",
    "MDockWidget",
    "MDrawer",
    "MFieldMixin",
    "MFlowLayout",
    "MForm",
    "MHeaderView",
    "MSortFilterModel",
    "MTableModel",
    "MAbstractView",
    "MBigView",
    "MListView",
    "MOptionDelegate",
    "MTableView",
    "MTreeView",
    "MItemViewFullSet",
    "MItemViewSet",
    "MLabel",
    "MLineEdit",
    "MLineTabWidget",
    "MUnderlineButton",
    "MUnderlineButtonGroup",
    "MLoading",
    "MLoadingWrapper",
    "MMenu",
    "MBlockButton",
    "MBlockButtonGroup",
    "MMenuTabWidget",
    "MMessage",
    "MPage",
    "MPopup",
    "MProgressBar",
    "MProgressCircle",
    "MPushButton",
    "MRadioButton",
    "MSlider",
    "MDateEdit",
    "MDateTimeEdit",
    "MDoubleSpinBox",
    "MSpinBox",
    "MTimeEdit",
    "MSplitter",
    "MStackedWidget",
    "MSwitch",
    "MTabBar",
    "MTabWidget",
    "MTextEdit",
    "MTheme",
    "MToast",
    "MToolButton",
    "MWidget",
]



import functools

from qtpy import QtCore, QtGui, QtWidgets

from .divider import MDivider
from .label import MLabel
from .message import MMessage
from .utils import generate_color


class MColorChart(QtWidgets.QWidget):
    def __init__(self, parent: QtWidgets.QWidget | None = None):
        super().__init__(parent)
        main_layout = QtWidgets.QVBoxLayout()
        main_layout.setSpacing(0)
        self.button_list: list[QtWidgets.QPushButton] = []
        for index in range(10):
            button = QtWidgets.QPushButton()
            button.setCursor(QtCore.Qt.CursorShape.PointingHandCursor)
            button.setToolTip("Click to Copy Color")
            button.setFixedSize(QtCore.QSize(250, 45))
            button.setText(f"color-{index + 1}")
            button.clicked.connect(functools.partial(self.on_copy_color, button))
            self.button_list.append(button)
            main_layout.addWidget(button)
        self.setLayout(main_layout)

    def set_colors(self, color_list: list[str]):
        for index, button in enumerate(self.button_list):
            target = color_list[index]
            button.setText(f"color-{index + 1}\t{target}")
            button.setProperty("color", target)
            button.setStyleSheet(
                "QPushButton{{background-color:{};color:{};border: 0 solid black}}"
                "QPushButton:hover{{font-weight:bold;}}".format(
                    target, "#000" if index < 5 else "#fff"
                )
            )

    def on_copy_color(self, button: QtWidgets.QPushButton):
        color = button.property("color")
        QtWidgets.QApplication.clipboard().setText(color)
        MMessage.success(f"copied: {color}", parent=self)


class MColorPaletteDialog(QtWidgets.QDialog):
    def __init__(self, init_color: str, parent: QtWidgets.QWidget | None = None):
        super().__init__(parent)
        self.setWindowTitle("Color Palette")

        self.primary_color = QtGui.QColor(init_color)
        self.color_chart = MColorChart()

        self.choose_color_button = QtWidgets.QPushButton()
        self.choose_color_button.setFixedSize(QtCore.QSize(100, 30))

        self.color_label = QtWidgets.QLabel()
        self.info_label = MLabel()
        self.info_label.setProperty("error", True)

        color_layout = QtWidgets.QHBoxLayout()
        color_layout.addWidget(MLabel("Primary Color:"))
        color_layout.addWidget(self.choose_color_button)
        color_layout.addWidget(self.color_label)
        color_layout.addWidget(self.info_label)
        color_layout.addStretch()

        dialog = QtWidgets.QColorDialog(self.primary_color, parent=self)
        dialog.setWindowFlags(QtCore.Qt.WindowType.Widget)
        dialog.setOption(QtWidgets.QColorDialog.ColorDialogOption.NoButtons)
        dialog.currentColorChanged.connect(self.on_color_changed)
        setting_layout = QtWidgets.QVBoxLayout()
        setting_layout.addLayout(color_layout)
        setting_layout.addWidget(MDivider())
        setting_layout.addWidget(dialog)

        main_layout = QtWidgets.QHBoxLayout()
        main_layout.addWidget(self.color_chart)
        main_layout.addLayout(setting_layout)
        self.setLayout(main_layout)
        self.update_color()

    @QtCore.Slot(QtGui.QColor)
    def on_color_changed(self, color: QtGui.QColor):
        self.primary_color = color
        light = self.primary_color.lightness()
        saturation = self.primary_color.saturation()
        self.info_label.setText("")
        if light <= 70:
            self.info_label.setText(f"lightness: {light}")
        if saturation <= 70:
            self.info_label.setText(f"saturation: {saturation}")

        self.update_color()

    def update_color(self):
        self.choose_color_button.setStyleSheet(
            "border-radius: 0;border: none;border:1px solid gray;"
            f"background-color:{self.primary_color.name()};"
        )
        self.color_label.setText(self.primary_color.name())
        self.color_chart.set_colors(
            [generate_color(self.primary_color, index + 1) for index in range(10)]
        )


if __name__ == "__main__":
    # Import built-in modules
    import sys

    # Import local modules
    from .theme import MTheme

    app = QtWidgets.QApplication(sys.argv)
    test = MColorPaletteDialog(init_color="#1890ff")
    MTheme().apply(test)
    test.show()
    sys.exit(app.exec_())



import functools
from collections.abc import Callable

from qtpy import QtWidgets

from qtui.dayu.breadcrumb import MBreadcrumb
from qtui.dayu.divider import MDivider
from qtui.dayu.field_mixin import MFieldMixin
from qtui.dayu.message import MMessage
from qtui.dayu.types import MBreadcrumbData


class BreadcrumbExample(QtWidgets.QWidget, MFieldMixin):
    def __init__(self, parent: QtWidgets.QWidget | None = None):
        super().__init__(parent)
        self.setWindowTitle("Examples for MBreadcrumb")
        self._init_ui()

    def _init_ui(self):
        MMessage.config(duration=1)
        entity_list = [
            MBreadcrumbData(
                clicked=functools.partial(
                    self.slot_show_message, MMessage.info, 'Go to "Home Page"'
                ),
                svg="home_line.svg",
            ),
            MBreadcrumbData(
                text="pl",
                clicked=functools.partial(
                    self.slot_show_message, MMessage.info, 'Go to "pl"'
                ),
                svg="user_line.svg",
            ),
            MBreadcrumbData(
                text="pl_0010",
                clicked=functools.partial(
                    self.slot_show_message, MMessage.info, 'Go to "pl_0010"'
                ),
            ),
        ]
        no_icon_eg = MBreadcrumb()
        no_icon_eg.set_item_list(entity_list)

        separator_eg = MBreadcrumb(separator="=>")
        separator_eg.set_item_list(entity_list)

        main_lay = QtWidgets.QVBoxLayout()
        main_lay.addWidget(MDivider("normal"))
        main_lay.addWidget(no_icon_eg)
        main_lay.addWidget(MDivider("separator: =>"))
        main_lay.addWidget(separator_eg)

        main_lay.addStretch()
        self.setLayout(main_lay)

    def slot_show_message(
        self, func: Callable[[str, QtWidgets.QWidget], MMessage], config: str
    ):
        func(config, self)


if __name__ == "__main__":
    from qtui.dayu.qt import application
    from qtui.dayu.theme import MTheme

    with application() as app:
        test = BreadcrumbExample()
        MTheme().apply(test)
        test.show()



from qtpy import QtWidgets

from qtui.dayu.divider import MDivider
from qtui.dayu.field_mixin import MFieldMixin
from qtui.dayu.push_button import MPushButton
from qtui.dayu.qt import MIcon
from qtui.dayu.theme import MTheme


class PushButtonExample(QtWidgets.QWidget, MFieldMixin):
    def __init__(self, parent: QtWidgets.QWidget | None = None):
        super().__init__(parent)
        self.setWindowTitle("Example for MPushButton")

        sub_lay1 = QtWidgets.QHBoxLayout()
        sub_lay1.addWidget(MPushButton("Default"))
        sub_lay1.addWidget(MPushButton("Primary").primary())
        sub_lay1.addWidget(MPushButton("Success").success())
        sub_lay1.addWidget(MPushButton("Warning").warning())
        sub_lay1.addWidget(MPushButton("Danger").danger())

        sub_lay2 = QtWidgets.QHBoxLayout()
        sub_lay2.addWidget(MPushButton("Upload", MIcon("cloud_line.svg")))
        sub_lay2.addWidget(
            MPushButton("Submit", MIcon("folder_line.svg", "#ddd")).primary()
        )
        sub_lay2.addWidget(
            MPushButton("Submit", MIcon("success_line.svg", "#ddd")).success()
        )
        sub_lay2.addWidget(
            MPushButton("Edit", MIcon("edit_line.svg", "#ddd")).warning()
        )
        sub_lay2.addWidget(
            MPushButton("Delete", MIcon("trash_line.svg", "#ddd")).danger()
        )

        sub_lay3 = QtWidgets.QHBoxLayout()
        sub_lay3.addWidget(MPushButton("Large").large().primary())
        sub_lay3.addWidget(MPushButton("Medium").medium().primary())
        sub_lay3.addWidget(MPushButton("Small").small().primary())

        disabled_button_1 = MPushButton("Disabled").huge()
        disabled_button_1.setEnabled(False)
        disabled_button_2 = MPushButton("Disabled").large()
        disabled_button_2.setEnabled(False)
        disabled_button_3 = MPushButton("Disabled")
        disabled_button_3.setEnabled(False)
        disabled_button_4 = MPushButton("Disabled").small()
        disabled_button_4.setEnabled(False)
        disable_lay = QtWidgets.QHBoxLayout()
        disable_lay.addWidget(disabled_button_1)
        disable_lay.addWidget(disabled_button_2)
        disable_lay.addWidget(disabled_button_3)
        disable_lay.addWidget(disabled_button_4)

        main_lay = QtWidgets.QVBoxLayout()
        main_lay.addWidget(MDivider("different type"))
        main_lay.addLayout(sub_lay1)
        main_lay.addLayout(sub_lay2)
        main_lay.addWidget(MDivider("different size"))
        main_lay.addLayout(sub_lay3)
        main_lay.addWidget(MDivider("disabled"))
        main_lay.addLayout(disable_lay)
        main_lay.addStretch()
        self.setLayout(main_lay)


if __name__ == "__main__":
    from qtui.dayu.qt import application
    from qtui.dayu.theme import MTheme

    with application() as app:
        test = PushButtonExample()
        MTheme().apply(test)
        test.show()



import functools

from qtpy import QtCore, QtWidgets

from qtui.dayu.badge import MBadge
from qtui.dayu.label import MLabel
from qtui.dayu.menu_tab_widget import MMenuTabWidget
from qtui.dayu.message import MMessage
from qtui.dayu.theme import MTheme
from qtui.dayu.tool_button import MToolButton
from qtui.dayu.types import MButtonGroupData


class MenuTabWidgetExample(QtWidgets.QWidget):
    def __init__(self, parent: QtWidgets.QWidget | None = None):
        super().__init__(parent)
        self.setWindowTitle("Examples for MMenuTabWidget")
        self._init_ui()

    def _init_ui(self):
        item_list = [
            MButtonGroupData(
                text="Overview",
                svg="home_line.svg",
                clicked=lambda: MMessage.info("Home", parent=self),
            ),
            MButtonGroupData(
                text="My",
                svg="user_line.svg",
                clicked=functools.partial(MMessage.info, "Edit Account", parent=self),
            ),
            MButtonGroupData(
                text="Notice",
                svg="alert_line.svg",
                clicked=functools.partial(MMessage.info, "Notice", parent=self),
            ),
        ]
        tool_bar = MMenuTabWidget()
        tool_bar_huge = MMenuTabWidget()
        tool_bar_huge.set_dayu_size(MTheme().huge)
        tool_bar_huge_v = MMenuTabWidget(orientation=QtCore.Qt.Orientation.Vertical)
        tool_bar_huge_v.set_dayu_size(MTheme().huge)
        tool_bar.tool_bar_insert_widget(MLabel("DaYu").h4().secondary().strong())
        tool_bar_huge.tool_bar_insert_widget(MLabel("DaYu").h4().secondary().strong())
        dayu_icon = MLabel("DaYu").h4().secondary().strong()
        dayu_icon.setContentsMargins(10, 10, 10, 10)
        tool_bar_huge_v.tool_bar_insert_widget(dayu_icon)
        tool_bar.tool_bar_append_widget(
            MBadge.dot(
                show=True, widget=MToolButton().icon_only().svg("user_fill.svg").large()
            )
        )
        for index, data_dict in enumerate(item_list):
            tool_bar.add_menu(data_dict, index)
            tool_bar_huge.add_menu(data_dict, index)
            tool_bar_huge_v.add_menu(data_dict, index)

        tool_bar.tool_button_group.set_dayu_checked(0)
        tool_bar_huge.tool_button_group.set_dayu_checked(0)
        tool_bar_huge_v.tool_button_group.set_dayu_checked(0)

        main_lay = QtWidgets.QVBoxLayout()
        main_lay.setContentsMargins(0, 0, 0, 0)

        main_lay.addWidget(MLabel("Menu Tab Widget (Large)"))
        main_lay.addWidget(tool_bar)

        main_lay.addWidget(MLabel("Menu Tab Widget (Huge)"))
        main_lay.addWidget(tool_bar_huge)

        main_lay.addWidget(MLabel("Menu Vertical Tab Widget (Huge)"))
        main_lay.addWidget(tool_bar_huge_v)

        self.setLayout(main_lay)


if __name__ == "__main__":
    from qtui.dayu.qt import application
    from qtui.dayu.theme import MTheme

    with application() as app:
        test = MenuTabWidgetExample()
        MTheme().apply(test)
        test.show()



"""MToolButton"""

from qtpy import QtCore, QtGui, QtWidgets

from .mixin import cursor_mixin
from .qt import MIcon
from .theme import MTheme
from .widget import MWidget


@cursor_mixin
class MToolButton(QtWidgets.QToolButton, MWidget):
    """MToolButton"""

    def __init__(self, parent: QtWidgets.QWidget | None = None):
        super().__init__(parent=parent)
        self._svg: str = ""
        self.setAutoExclusive(False)
        self.setAutoRaise(True)

        self._polish_icon()
        self.toggled.connect(self._polish_icon)
        self.setSizePolicy(
            QtWidgets.QSizePolicy.Policy.Minimum, QtWidgets.QSizePolicy.Policy.Minimum
        )

    @QtCore.Slot()
    def _polish_icon(self):
        if self._svg:
            if self.isCheckable() and self.isChecked():
                self.setIcon(MIcon(self._svg, MTheme().primary_color))
            else:
                self.setIcon(MIcon(self._svg))

    def enterEvent(self, event: QtGui.QEnterEvent):  # noqa: N802
        """Override enter event to highlight the icon"""
        if self._svg:
            self.setIcon(MIcon(self._svg, MTheme().primary_color))
        return super().enterEvent(event)

    def leaveEvent(self, event: QtCore.QEvent):  # noqa: N802
        """Override leave event to recover the icon"""
        self._polish_icon()
        return super().leaveEvent(event)

    def get_dayu_size(self) -> int:
        """
        Get the push button height
        :return: integer
        """
        return self._dayu_size

    def set_dayu_size(self, value: int):
        """
        Set the avatar size.
        :param value: integer
        :return: None
        """
        if value != self._dayu_size:
            super().set_dayu_size(value)
            if self.toolButtonStyle() == QtCore.Qt.ToolButtonStyle.ToolButtonIconOnly:
                self.setFixedSize(QtCore.QSize(self._dayu_size, self._dayu_size))

    def get_svg(self):
        """Get current svg path"""
        return self._svg

    def set_svg(self, name: str):
        """Set current svg path"""
        self._svg = name
        self._polish_icon()

    dayu_size = QtCore.Property(int, get_dayu_size, set_dayu_size)

    def huge(self):
        """Set MPushButton to PrimaryType"""
        self.set_dayu_size(MTheme().huge)
        return self

    def large(self):
        """Set MPushButton to SuccessType"""
        self.set_dayu_size(MTheme().large)
        return self

    def medium(self):
        """Set MPushButton to  WarningType"""
        self.set_dayu_size(MTheme().medium)
        return self

    def small(self):
        """Set MPushButton to DangerType"""
        self.set_dayu_size(MTheme().small)
        return self

    def tiny(self):
        """Set MPushButton to DangerType"""
        self.set_dayu_size(MTheme().tiny)
        return self

    def svg(self, name: str):
        """Set current svg path"""
        self.set_svg(name)
        return self

    def icon_only(self):
        """Set tool button style to icon only"""
        self.setToolButtonStyle(QtCore.Qt.ToolButtonStyle.ToolButtonIconOnly)
        self.setFixedSize(QtCore.QSize(self._dayu_size, self._dayu_size))
        return self

    def text_only(self):
        """Set tool button style to text only"""
        self.setToolButtonStyle(QtCore.Qt.ToolButtonStyle.ToolButtonTextOnly)
        return self

    def text_beside_icon(self):
        """Set tool button style to text beside icon"""
        self.setToolButtonStyle(QtCore.Qt.ToolButtonStyle.ToolButtonTextBesideIcon)
        return self

    def text_under_icon(self):
        """Set tool button style to text under icon"""
        self.setToolButtonStyle(QtCore.Qt.ToolButtonStyle.ToolButtonTextUnderIcon)
        return self



import re
from collections.abc import Iterator
from typing import Any

from qtpy import QtCore, QtGui

from .types import HeaderData, ModelData
from .utils import (
    apply_formatter,
    display_formatter,
    font_formatter,
    get_obj_value,
    icon_formatter,
    set_obj_value,
)

SETTING_MAP: dict[QtCore.Qt.ItemDataRole, dict[str, Any]] = {
    QtCore.Qt.ItemDataRole.BackgroundRole: {
        "config": "bg_color",
        "formatter": QtGui.QColor,
    },
    QtCore.Qt.ItemDataRole.DisplayRole: {
        "config": "display",
        "formatter": display_formatter,
    },
    QtCore.Qt.ItemDataRole.EditRole: {"config": "edit", "formatter": None},
    QtCore.Qt.ItemDataRole.TextAlignmentRole: {
        "config": "alignment",
        "formatter": {
            "right": QtCore.Qt.AlignmentFlag.AlignRight,
            "left": QtCore.Qt.AlignmentFlag.AlignLeft,
            "center": QtCore.Qt.AlignmentFlag.AlignCenter,
        },
    },
    QtCore.Qt.ItemDataRole.ForegroundRole: {
        "config": "color",
        "formatter": QtGui.QColor,
    },
    QtCore.Qt.ItemDataRole.FontRole: {"config": "font", "formatter": font_formatter},
    QtCore.Qt.ItemDataRole.DecorationRole: {
        "config": "icon",
        "formatter": icon_formatter,
    },
    QtCore.Qt.ItemDataRole.ToolTipRole: {
        "config": "tooltip",
        "formatter": display_formatter,
    },
    QtCore.Qt.ItemDataRole.InitialSortOrderRole: {
        "config": "order",
        "formatter": {
            "asc": QtCore.Qt.SortOrder.AscendingOrder,
            "des": QtCore.Qt.SortOrder.DescendingOrder,
        },
    },
    QtCore.Qt.ItemDataRole.SizeHintRole: {
        "config": "size",
        "formatter": lambda args: QtCore.QSize(*args),  # pyright: ignore[reportUnknownLambdaType]
    },
    QtCore.Qt.ItemDataRole.UserRole: {"config": "data"},  # anything
}


class MTableModel(QtCore.QAbstractItemModel):
    def __init__(self, parent: QtCore.QObject | None = None):
        super().__init__(parent)
        self.origin_count = 0
        self.root_item: ModelData = {"name": "root", "children": []}
        self.data_generator: Iterator[ModelData] | None = None
        self.header_list: list[HeaderData] = []
        self.timer = QtCore.QTimer(self)
        self.timer.timeout.connect(self.fetchMore)

    def set_header_list(self, header_list: list[HeaderData]):
        self.header_list = header_list

    def set_data_list(self, data_list: list[ModelData] | Iterator[ModelData] | None):
        if isinstance(data_list, Iterator):
            self.beginResetModel()
            self.root_item["children"] = []
            self.endResetModel()
            self.data_generator = data_list
            self.origin_count = 0
            self.timer.start()
        else:
            self.beginResetModel()
            self.root_item["children"] = data_list if data_list is not None else []
            self.endResetModel()
            self.data_generator = None

    def clear(self):
        self.beginResetModel()
        self.root_item["children"] = []
        self.endResetModel()

    def get_data_list(self):
        return self.root_item["children"]

    def append(self, data_dict: ModelData):
        self.root_item["children"].append(data_dict)
        self.fetchMore()

    def remove(self, data_dict: ModelData):
        row = self.root_item["children"].index(data_dict)
        self.beginRemoveRows(QtCore.QModelIndex(), row, row)
        self.root_item["children"].remove(data_dict)
        self.endRemoveRows()

    def flags(self, index: QtCore.QModelIndex) -> QtCore.Qt.ItemFlag:  # pyright: ignore[reportIncompatibleMethodOverride]
        result = QtCore.QAbstractItemModel.flags(self, index)
        if not index.isValid():
            return QtCore.Qt.ItemFlag.ItemIsEnabled
        if self.header_list[index.column()].get("checkable", False):
            result |= QtCore.Qt.ItemFlag.ItemIsUserCheckable
        if self.header_list[index.column()].get("selectable", False):
            result |= QtCore.Qt.ItemFlag.ItemIsEditable
        if self.header_list[index.column()].get("editable", False):
            result |= QtCore.Qt.ItemFlag.ItemIsEditable
        if self.header_list[index.column()].get("draggable", False):
            result |= QtCore.Qt.ItemFlag.ItemIsDragEnabled
        if self.header_list[index.column()].get("droppable", False):
            result |= QtCore.Qt.ItemFlag.ItemIsDropEnabled
        return QtCore.Qt.ItemFlag(result)

    def headerData(  # noqa: N802  # pyright: ignore[reportIncompatibleMethodOverride]
        self,
        section: int,
        orientation: QtCore.Qt.Orientation,
        role: QtCore.Qt.ItemDataRole = QtCore.Qt.ItemDataRole.DisplayRole,
    ) -> Any:
        if orientation == QtCore.Qt.Orientation.Vertical:
            return super().headerData(section, orientation, role)
        if not self.header_list or section >= len(self.header_list):
            return None
        if role == QtCore.Qt.ItemDataRole.DisplayRole:
            return self.header_list[section]["label"]
        return None

    def index(  # pyright: ignore[reportIncompatibleMethodOverride]
        self, row: int, column: int, parent_index: QtCore.QModelIndex | None = None
    ) -> QtCore.QModelIndex:
        if parent_index and parent_index.isValid():
            parent_item = parent_index.internalPointer()
        else:
            parent_item = self.root_item

        children_list = get_obj_value(parent_item, "children")
        if children_list and len(children_list) > row:
            child_item = children_list[row]
            if child_item:
                set_obj_value(child_item, "_parent", parent_item)
                return self.createIndex(row, column, child_item)
        return QtCore.QModelIndex()

    def parent(self, index: QtCore.QModelIndex) -> QtCore.QModelIndex:  # pyright: ignore[reportIncompatibleMethodOverride]
        if not index.isValid():
            return QtCore.QModelIndex()

        child_item = index.internalPointer()
        parent_item = get_obj_value(child_item, "_parent")

        if parent_item is None:
            return QtCore.QModelIndex()

        grand_item = get_obj_value(parent_item, "_parent")
        if grand_item is None:
            return QtCore.QModelIndex()
        parent_list = get_obj_value(grand_item, "children")
        return self.createIndex(parent_list.index(parent_item), 0, parent_item)

    def rowCount(self, parent_index: QtCore.QModelIndex | None = None) -> int:  # noqa: N802  # pyright: ignore[reportIncompatibleMethodOverride]
        if parent_index and parent_index.isValid():
            parent_item = parent_index.internalPointer()
        else:
            parent_item = self.root_item
        children_obj = get_obj_value(parent_item, "children")
        if hasattr(children_obj, "next") or (children_obj is None):
            return 0
        else:
            return len(children_obj)

    def hasChildren(self, parent_index: QtCore.QModelIndex | None = None) -> bool | int:  # noqa: N802  # pyright: ignore[reportIncompatibleMethodOverride]
        if parent_index and parent_index.isValid():
            parent_data = parent_index.internalPointer()
        else:
            parent_data = self.root_item
        children_obj = get_obj_value(parent_data, "children")
        if children_obj is None:
            return False
        if hasattr(children_obj, "__next__"):
            return True
        else:
            return len(children_obj)

    def columnCount(self, parent_index: QtCore.QModelIndex | None = None) -> int:  # noqa: N802  # pyright: ignore[reportIncompatibleMethodOverride]
        return len(self.header_list)

    def canFetchMore(self, index: QtCore.QModelIndex | None = None) -> bool:  # noqa: N802  # pyright: ignore[reportIncompatibleMethodOverride]
        try:
            if self.data_generator:
                data = next(self.data_generator)
                self.root_item["children"].append(data)
                return True
            return False
        except StopIteration:
            if self.timer.isActive():
                self.timer.stop()
            return False

    def fetchMore(self, index: QtCore.QModelIndex | None = None) -> None:  # noqa: N802  # pyright: ignore[reportIncompatibleMethodOverride]
        self.beginResetModel()
        self.endResetModel()

    def data(  # pyright: ignore[reportIncompatibleMethodOverride]
        self,
        index: QtCore.QModelIndex,
        role: QtCore.Qt.ItemDataRole = QtCore.Qt.ItemDataRole.DisplayRole,
    ) -> Any:
        if not index.isValid():
            return None

        attr_dict = self.header_list[index.column()]
        data_obj = index.internalPointer()
        attr = attr_dict.get("key")
        if role in SETTING_MAP:
            role_key = SETTING_MAP[role].get("config")
            formatter_from_config = attr_dict.get(role_key)  # pyright: ignore[reportCallIssue, reportArgumentType, reportUnknownVariableType]
            if not formatter_from_config and role not in [
                QtCore.Qt.ItemDataRole.DisplayRole,
                QtCore.Qt.ItemDataRole.EditRole,
                QtCore.Qt.ItemDataRole.ToolTipRole,
            ]:
                return None
            else:
                value = apply_formatter(
                    formatter_from_config, get_obj_value(data_obj, attr), data_obj
                )
            formatter_from_model = SETTING_MAP[role].get("formatter", None)
            result = apply_formatter(formatter_from_model, value)
            return result

        if role == QtCore.Qt.ItemDataRole.CheckStateRole and attr_dict.get(
            "checkable", False
        ):
            state = get_obj_value(data_obj, attr + "_checked")
            return QtCore.Qt.CheckState.Unchecked if state is None else state
        return None

    def setData(  # noqa: N802  # pyright: ignore[reportIncompatibleMethodOverride]
        self,
        index: QtCore.QModelIndex,
        value: Any,
        role: QtCore.Qt.ItemDataRole = QtCore.Qt.ItemDataRole.EditRole,
    ) -> bool:
        if index.isValid() and role in [
            QtCore.Qt.ItemDataRole.CheckStateRole,
            QtCore.Qt.ItemDataRole.EditRole,
        ]:
            attr_dict = self.header_list[index.column()]
            key = attr_dict.get("key")
            data_obj = index.internalPointer()
            if role == QtCore.Qt.ItemDataRole.CheckStateRole and attr_dict.get(
                "checkable", False
            ):
                key += "_checked"
                set_obj_value(data_obj, key, value)
                self.dataChanged.emit(index, index)

                for row, sub_obj in enumerate(get_obj_value(data_obj, "children", [])):
                    set_obj_value(sub_obj, key, value)
                    sub_index = self.index(row, index.column(), index)
                    self.dataChanged.emit(sub_index, sub_index)

                parent_index = index.parent()
                if parent_index.isValid():
                    parent_obj = parent_index.internalPointer()
                    new_parent_value = value
                    old_parent_value = get_obj_value(parent_obj, key)
                    for sibling_obj in get_obj_value(
                        get_obj_value(data_obj, "_parent"), "children", []
                    ):
                        if value != get_obj_value(sibling_obj, key):
                            new_parent_value = 1
                            break
                    if new_parent_value != old_parent_value:
                        set_obj_value(parent_obj, key, new_parent_value)
                        self.dataChanged.emit(parent_index, parent_index)
            else:
                set_obj_value(data_obj, key, value)
                self.dataChanged.emit(index, index)
            return True
        else:
            return False


class MSortFilterModel(QtCore.QSortFilterProxyModel):
    def __init__(self, parent: QtCore.QObject | None = None):
        super().__init__(parent)
        if hasattr(self, "setRecursiveFilteringEnabled"):
            self.setRecursiveFilteringEnabled(True)
        self.header_list: list[HeaderData] = []
        self.search_reg: re.Pattern[str] | None = None

    def set_header_list(self, header_list: list[HeaderData]):
        self.header_list = header_list
        for head in self.header_list:
            head.update({"reg": None})

    def filterAcceptsRow(  # noqa: N802  # pyright: ignore[reportIncompatibleMethodOverride]
        self, source_row: int, source_parent: QtCore.QModelIndex
    ) -> bool:
        if self.search_reg:
            for index, data_dict in enumerate(self.header_list):
                if data_dict.get("searchable", False):
                    model_index = self.sourceModel().index(
                        source_row, index, source_parent
                    )
                    value = self.sourceModel().data(model_index)
                    if self.search_reg.match(str(value)) is not None:
                        break
            else:
                return False

        for index, data_dict in enumerate(self.header_list):
            model_index = self.sourceModel().index(source_row, index, source_parent)
            value = self.sourceModel().data(model_index)
            reg_exp = data_dict.get("reg", None)
            if reg_exp and (not reg_exp.match(value)):
                return False

        return True

    def set_search_pattern(self, pattern: str):
        self.search_reg = re.compile(pattern)
        self.invalidateFilter()

    def set_filter_attr_pattern(self, attr: str, pattern: str):
        for data_dict in self.header_list:
            if data_dict.get("key") == attr:
                data_dict["reg"] = re.compile(pattern)
                break
        self.invalidateFilter()



"""
MBadge
"""

from qtpy import QtCore, QtWidgets

from .utils import overflow_format
from .widget import MWidget


class MBadge(QtWidgets.QWidget, MWidget):  # pyright: ignore[reportGeneralTypeIssues]
    """
    Badge normally appears in proximity to notifications
    or user avatars with eye-catching appeal,
    typically displaying unread messages count.
    Show something at the wrapped widget top right.

    There is 3 type styles:
        dot: show a dot
        count: show a number at
        text: show a string

    Property:
        dayu_dot: bool
        dayu_text: str
        dayu_count: int
        dayu_overflow: int
    """

    def __init__(
        self,
        widget: QtWidgets.QWidget | None = None,
        parent: QtWidgets.QWidget | None = None,
    ):
        super().__init__(parent)
        self._widget = widget
        self._overflow_count: int = 99
        self._dot: bool = False
        self._text: str = ""
        self._count: int = 0
        self._badge_button = QtWidgets.QPushButton()
        self._badge_button.setSizePolicy(
            QtWidgets.QSizePolicy.Policy.Minimum, QtWidgets.QSizePolicy.Policy.Minimum
        )
        self._main_layout = QtWidgets.QGridLayout()
        self._main_layout.setContentsMargins(0, 0, 0, 0)

        if widget is not None:
            self._main_layout.addWidget(widget, 0, 0)

        self._main_layout.addWidget(
            self._badge_button,
            0,
            0,
            QtCore.Qt.AlignmentFlag.AlignTop | QtCore.Qt.AlignmentFlag.AlignRight,
        )
        self.setLayout(self._main_layout)

    def get_dayu_overflow(self):
        """
        Get current overflow number
        :return: int
        """
        return self._overflow_count

    def set_dayu_overflow(self, num: int):
        """
        Set the overflow number
        :param num: new max number
        :return: None
        """
        if num != self._overflow_count:
            self._overflow_count = num
            self._update_number()

    def get_dayu_dot(self):
        """
        Get current style is dot or not and dot is show or not
        :return: bool
        """
        return self._dot

    def set_dayu_dot(self, show: bool):
        """
        Set dot style and weather show the dot or not
        :param show: bool
        :return: None
        """
        if show != self._dot:
            self._dot = show
            self._badge_button.setText("")
            self._badge_button.setVisible(show)
            self.on_style_changed()

    def get_dayu_count(self):
        """
        Get actual count number
        :return: int
        """
        return self._count

    def set_dayu_count(self, num: int):
        """
        Set current style to show a number
        :param num: int
        :return: None
        """
        if num != self._count:
            self._count = num
            self._update_number()

    def _update_number(self):
        self._badge_button.setText(overflow_format(self._count, self._overflow_count))
        self._badge_button.setVisible(self._count > 0)
        self._dot = False
        self.on_style_changed()

    def get_dayu_text(self):
        """
        Get current showed text
        :return: str
        """

        return self._text

    def set_dayu_text(self, text: str):
        """
        Set current style to show a text.
        :param text: str
        :return: None
        """
        if text != self._text:
            self._text = text
            self._badge_button.setText(self._text)
            self._badge_button.setVisible(bool(self._text))
            self._dot = False
            self.on_style_changed()

    dayu_overflow = QtCore.Property(int, get_dayu_overflow, set_dayu_overflow)
    dayu_dot = QtCore.Property(bool, get_dayu_dot, set_dayu_dot)
    dayu_count = QtCore.Property(int, get_dayu_count, set_dayu_count)
    dayu_text = QtCore.Property(str, get_dayu_text, set_dayu_text)

    @classmethod
    def dot(cls, show: bool = False, widget: QtWidgets.QWidget | None = None):
        """
        Create a Badge with dot style.
        :param show: bool
        :param widget: the wrapped widget
        :return: instance badge
        """
        inst = cls(widget=widget)
        inst.set_dayu_dot(show)
        return inst

    @classmethod
    def count(cls, count: int = 0, widget: QtWidgets.QWidget | None = None):
        """
        Create a Badge with number style.
        :param count: int
        :param widget: the wrapped widget
        :return: instance badge
        """
        inst = cls(widget=widget)
        inst.set_dayu_count(count)
        return inst

    @classmethod
    def text(cls, text: str = "", widget: QtWidgets.QWidget | None = None):
        """
        Create a Badge with text style.
        :param text: str
        :param widget: the wrapped widget
        :return: instance badge
        """
        inst = cls(widget=widget)
        inst.set_dayu_text(text)
        return inst



from qtpy import QtCore, QtWidgets

from qtui.dayu.button_group import MPushButtonGroup
from qtui.dayu.divider import MDivider
from qtui.dayu.field_mixin import MFieldMixin
from qtui.dayu.label import MLabel
from qtui.dayu.push_button import MPushButton
from qtui.dayu.qt import MIcon
from qtui.dayu.theme import MTheme
from qtui.dayu.types import MButtonGroupData


class PushButtonGroupExample(QtWidgets.QWidget, MFieldMixin):
    def __init__(self, parent: QtWidgets.QWidget | None = None):
        super().__init__(parent)
        self._init_ui()

    def _init_ui(self):
        button_config_list = [
            MButtonGroupData(
                text="Add",
                icon=MIcon("add_line.svg", "#fff"),
                dayu_type=MPushButton.ButtonType.Primary,
            ),
            MButtonGroupData(
                text="Edit",
                icon=MIcon("edit_fill.svg", "#fff"),
                dayu_type=MPushButton.ButtonType.Warning,
            ),
            MButtonGroupData(
                text="Delete",
                icon=MIcon("trash_line.svg", "#fff"),
                dayu_type=MPushButton.ButtonType.Danger,
            ),
        ]
        button_group_h = MPushButtonGroup()
        button_group_h.set_dayu_size(MTheme().large)
        button_group_h.set_button_list(button_config_list)
        h_lay = QtWidgets.QHBoxLayout()
        h_lay.addWidget(button_group_h)
        h_lay.addStretch()

        button_group_v = MPushButtonGroup(orientation=QtCore.Qt.Orientation.Vertical)
        button_group_v.set_button_list(button_config_list)
        h_lay_2 = QtWidgets.QHBoxLayout()
        h_lay_2.addWidget(button_group_v)
        h_lay_2.addStretch()

        main_lay = QtWidgets.QVBoxLayout()
        main_lay.addWidget(
            MLabel(
                "MPushButtonGroup is MPushButton collection. they are not exclusive."
            )
        )
        main_lay.addWidget(MDivider("MPushButton group: Horizontal & Small Size"))
        main_lay.addLayout(h_lay)
        main_lay.addWidget(MDivider("MPushButton group: Vertical & Default Size"))
        main_lay.addLayout(h_lay_2)
        main_lay.addStretch()
        self.setLayout(main_lay)


if __name__ == "__main__":
    from qtui.dayu.qt import application
    from qtui.dayu.theme import MTheme

    with application() as app:
        test = PushButtonGroupExample()
        MTheme().apply(test)
        test.show()



"""MSlider"""

# Import third-party modules
from qtpy import QtCore, QtGui, QtWidgets


class MSlider(QtWidgets.QSlider):
    """
    A Slider component for displaying current value and intervals in range.

    MSlider just apply qss for QSlider.
    """

    def __init__(
        self,
        orientation: QtCore.Qt.Orientation = QtCore.Qt.Orientation.Horizontal,
        parent: QtWidgets.QWidget | None = None,
    ):
        super().__init__(orientation, parent=parent)
        self._show_text_when_move = True

    def disable_show_text(self):
        self._show_text_when_move = False

    def mouseMoveEvent(self, event: QtGui.QMouseEvent):  # noqa: N802
        """Override the mouseMoveEvent to show current value as a tooltip."""
        if self._show_text_when_move:
            QtWidgets.QToolTip.showText(event.globalPos(), str(self.value()), self)
        return super().mouseMoveEvent(event)



from __future__ import annotations

from collections.abc import Callable
from typing import Any, cast

from qtpy import QtCore, QtWidgets

from . import utils
from .completer import MCompleter
from .menu import MMenu
from .mixin import cursor_mixin, focus_shadow_mixin, property_mixin
from .theme import MTheme
from .widget import MWidget


@property_mixin
@cursor_mixin
@focus_shadow_mixin
class MComboBox(QtWidgets.QComboBox, MWidget):
    value_changed = QtCore.Signal(object)

    def __init__(self, parent: QtWidgets.QWidget | None = None):
        super().__init__(parent)

        self.filter_model = QtCore.QSortFilterProxyModel(self)
        self.filter_model.setFilterCaseSensitivity(
            QtCore.Qt.CaseSensitivity.CaseInsensitive
        )
        self.filter_model.setSourceModel(self.model())

        self._completer = MCompleter(self)
        self._completer.setCompletionMode(
            QtWidgets.QCompleter.CompletionMode.UnfilteredPopupCompletion
        )
        self._completer.setModel(self.filter_model)

        self._root_menu: MMenu | None = None
        self._display_formatter = utils.display_formatter

        self.setEditable(True)
        self.line_edit = cast(QtWidgets.QLineEdit, self.lineEdit())
        self.line_edit.setReadOnly(True)
        self.line_edit.setTextMargins(4, 0, 4, 0)
        self.line_edit.setStyleSheet("background-color:transparent")
        self.line_edit.setCursor(QtCore.Qt.CursorShape.PointingHandCursor)
        self.line_edit.installEventFilter(self)
        self._has_custom_view: bool = False
        self.set_value("")
        self.set_placeholder("Please Select")
        self.setSizePolicy(
            QtWidgets.QSizePolicy.Policy.Expanding, QtWidgets.QSizePolicy.Policy.Minimum
        )
        self._dayu_size: int = MTheme().default_size
        MTheme().apply(self)

    def search(self):
        self.setFocusPolicy(QtCore.Qt.FocusPolicy.StrongFocus)
        self.setEditable(True)

        self.setCompleter(self._completer)

        self.line_edit.setReadOnly(False)
        self.line_edit.returnPressed.disconnect()
        self.line_edit.textEdited.connect(self.filter_model.setFilterFixedString)
        self._completer.activated.connect(self._completer_activated)

    def _completer_activated(self, text: str) -> None:
        if text:
            self.setCurrentIndex(self.findText(text))

    def _set_searchable(self, value: bool):
        """search property to True then trigger search"""
        if value:
            self.search()

    def setModel(self, model: QtCore.QAbstractItemModel):  # noqa: N802
        super().setModel(model)
        self.filter_model.setSourceModel(model)
        self._completer.setModel(self.filter_model)

    def setModelColumn(self, column: int):  # noqa: N802
        self._completer.setCompletionColumn(column)
        self.filter_model.setFilterKeyColumn(column)
        super().setModelColumn(column)

    def get_dayu_size(self) -> int:
        """
        Get the push button height
        :return: integer
        """
        return self._dayu_size

    def set_dayu_size(self, value: int):
        """
        Set the avatar size.
        :param value: integer
        :return: None
        """
        self._dayu_size = value
        self.line_edit.setProperty("dayu_size", value)
        self.style().polish(self)

    dayu_size = QtCore.Property(int, get_dayu_size, set_dayu_size)

    def set_formatter(self, func: Callable[[Any], str]):
        self._display_formatter = func

    def set_placeholder(self, text: str):
        """Display the text when no item selected."""
        self.line_edit.setPlaceholderText(text)

    def set_value(self, value: Any):
        self.setProperty("value", value)

    def _set_value(self, value: Any):
        self.line_edit.setProperty("text", self._display_formatter(value))
        if self._root_menu:
            self._root_menu.set_value(value)

    def set_menu(self, menu: MMenu):
        self._root_menu = menu
        self._root_menu.value_changed.connect(self.value_changed)
        self._root_menu.value_changed.connect(self.set_value)

    def setView(self, *args: Any, **kwargs: Any):  # noqa: N802
        """Override setView to flag _has_custom_view variable."""
        self._has_custom_view = True
        super().setView(*args, **kwargs)

    def showPopup(self) -> None:  # noqa: N802
        """Override default showPopup. When set custom menu, show the menu instead."""
        if self._has_custom_view or self._root_menu is None:
            super().showPopup()
        else:
            super().hidePopup()
            self._root_menu.popup(self.mapToGlobal(QtCore.QPoint(0, self.height())))

    def eventFilter(self, widget: QtWidgets.QWidget, event: QtCore.QEvent) -> bool:  # noqa: N802  # pyright: ignore[reportIncompatibleMethodOverride]
        if (
            widget is self.line_edit
            and self.line_edit.isReadOnly()
            and self.isEnabled()
            and event.type() == QtCore.QEvent.Type.MouseButtonPress
        ):
            self.showPopup()
        return super().eventFilter(widget, event)

    def huge(self):
        """Set MComboBox to huge size"""
        self.set_dayu_size(MTheme().huge)
        return self

    def large(self):
        """Set MComboBox to large size"""
        self.set_dayu_size(MTheme().large)
        return self

    def medium(self):
        """Set MComboBox to  medium"""
        self.set_dayu_size(MTheme().medium)
        return self

    def small(self):
        """Set MComboBox to small size"""
        self.set_dayu_size(MTheme().small)
        return self

    def tiny(self):
        """Set MComboBox to tiny size"""
        self.set_dayu_size(MTheme().tiny)
        return self



"""
MPushButton.
"""

from enum import StrEnum

from qtpy import QtGui, QtWidgets

from .mixin import cursor_mixin, focus_shadow_mixin
from .theme import MTheme
from .widget import MWidget


@cursor_mixin
@focus_shadow_mixin
class MPushButton(QtWidgets.QPushButton, MWidget):
    """
    QPushButton.
    """

    class ButtonType(StrEnum):
        Default = "default"
        Primary = "primary"
        Success = "success"
        Warning = "warning"
        Danger = "danger"

    def __init__(
        self,
        text: str = "",
        icon: QtGui.QIcon | None = None,
        parent: QtWidgets.QWidget | None = None,
    ):
        if icon is None:
            super().__init__(text, parent=parent)
        else:
            super().__init__(icon, text, parent=parent)
        self.set_dayu_type(MPushButton.ButtonType.Default)

    def primary(self):
        """Set MPushButton to PrimaryType"""
        self.set_dayu_type(MPushButton.ButtonType.Primary)
        return self

    def success(self):
        """Set MPushButton to SuccessType"""
        self.set_dayu_type(MPushButton.ButtonType.Success)
        return self

    def warning(self):
        """Set MPushButton to  WarningType"""
        self.set_dayu_type(MPushButton.ButtonType.Warning)
        return self

    def danger(self):
        """Set MPushButton to DangerType"""
        self.set_dayu_type(MPushButton.ButtonType.Danger)
        return self

    def huge(self):
        """Set MPushButton to huge size"""
        self.set_dayu_size(MTheme().huge)
        return self

    def large(self):
        """Set MPushButton to large size"""
        self.set_dayu_size(MTheme().large)
        return self

    def medium(self):
        """Set MPushButton to  medium"""
        self.set_dayu_size(MTheme().medium)
        return self

    def small(self):
        """Set MPushButton to small size"""
        self.set_dayu_size(MTheme().small)
        return self

    def tiny(self):
        """Set MPushButton to tiny size"""
        self.set_dayu_size(MTheme().tiny)
        return self



"""
Example code for MSplitter
"""

from qtpy import QtCore, QtWidgets

from qtui.dayu.splitter import MSplitter
from qtui.dayu.text_edit import MTextEdit
from qtui.dayu.theme import MTheme


class SplitterExample(QtWidgets.QWidget):
    def __init__(self, parent: QtWidgets.QWidget | None = None):
        super().__init__(parent)

        main_splitter = MSplitter(QtCore.Qt.Orientation.Vertical)
        main_splitter.setHandleWidth(20)

        container = QtWidgets.QWidget()
        layout = QtWidgets.QVBoxLayout()
        container.setLayout(layout)
        splitter = MSplitter(QtCore.Qt.Orientation.Vertical)
        splitter.addWidget(MTextEdit())
        splitter.addWidget(MTextEdit())
        splitter.addWidget(MTextEdit())
        layout.addWidget(splitter)
        main_splitter.addWidget(container)

        container = QtWidgets.QWidget()
        layout = QtWidgets.QVBoxLayout()
        container.setLayout(layout)
        splitter = MSplitter()
        splitter.addWidget(MTextEdit())
        splitter.addWidget(MTextEdit())
        splitter.addWidget(MTextEdit())
        layout.addWidget(splitter)
        main_splitter.addWidget(container)

        layout = QtWidgets.QVBoxLayout()
        layout.addWidget(main_splitter)
        self.setLayout(layout)

        self.resize(800, 800)


if __name__ == "__main__":
    from qtui.dayu.qt import application

    with application() as app:
        test = SplitterExample()
        MTheme().apply(test)
        test.show()



from qtpy import QtWidgets

from qtui.dayu.carousel import MCarousel
from qtui.dayu.label import MLabel
from qtui.dayu.qt import MPixmap
from qtui.dayu.slider import MSlider
from qtui.dayu.switch import MSwitch


class CarouselExample(QtWidgets.QWidget):
    def __init__(self, parent: QtWidgets.QWidget | None = None):
        super().__init__(parent)
        self.setWindowTitle("Examples for MCarousel")
        self._init_ui()

    def _init_ui(self):
        switch = MSwitch()
        switch.setChecked(True)
        slider = MSlider()
        slider.setRange(1, 10)
        switch_lay = QtWidgets.QFormLayout()
        switch_lay.addRow(MLabel("AutoPlay"), switch)
        switch_lay.addRow(MLabel("Interval"), slider)
        test = MCarousel(
            [MPixmap(f"app-{a}.png") for a in ["maya", "nuke", "houdini"]],
            width=300,
            height=300,
            autoplay=True,
        )
        switch.toggled.connect(test.set_autoplay)

        def set_interval(x: int):
            test.set_interval(x * 1000)

        slider.valueChanged.connect(set_interval)
        slider.setValue(3)

        main_lay = QtWidgets.QVBoxLayout()
        main_lay.addWidget(test)
        main_lay.addLayout(switch_lay)
        main_lay.addStretch()
        self.setLayout(main_lay)


if __name__ == "__main__":
    from qtui.dayu.qt import application
    from qtui.dayu.theme import MTheme

    with application() as app:
        test = CarouselExample()
        MTheme().apply(test)
        test.show()



from __future__ import annotations

from collections.abc import Callable

from qtpy import QtCore, QtGui, QtWidgets

from .avatar import MAvatar
from .divider import MDivider
from .label import MLabel
from .menu import MMenu
from .mixin import cursor_mixin, hover_shadow_mixin
from .theme import MTheme
from .tool_button import MToolButton
from .types import MCardTitleProps


@cursor_mixin
class MCard(QtWidgets.QWidget):
    def __init__(
        self,
        title: str = "",
        image: QtGui.QPixmap | None = None,
        size: int | None = None,
        extra: bool = False,
        extra_menus: list[tuple[str, Callable[[], None]]] | None = None,
        border: bool = False,
        parent: QtWidgets.QWidget | None = None,
    ):
        super().__init__(parent)
        self.setAttribute(QtCore.Qt.WidgetAttribute.WA_StyledBackground)
        self._title_label = MLabel().h4()
        self._title_icon = MAvatar()
        self._title_icon_pixmap: QtGui.QPixmap | None = image
        self._title_divider = MDivider()
        self._extra_button = MToolButton().icon_only().svg("more.svg")
        self._extra_menus = extra_menus or []
        self._extra_button.clicked.connect(self._on_extra_menu_clicked)

        self._title_layout = QtWidgets.QHBoxLayout()
        self._title_layout.setContentsMargins(10, 10, 10, 10)
        self._title_layout.addWidget(self._title_icon)
        self._title_layout.addStretch()
        self._title_layout.addWidget(self._title_label)
        self._title_layout.addWidget(self._extra_button)

        self._content_layout = QtWidgets.QVBoxLayout()
        self._main_layout = QtWidgets.QVBoxLayout()
        self._main_layout.setSpacing(0)
        self._main_layout.setContentsMargins(1, 1, 1, 1)
        self._main_layout.addLayout(self._title_layout)
        self._main_layout.addWidget(self._title_divider)
        self._main_layout.addLayout(self._content_layout)
        self.setLayout(self._main_layout)

        size = size or MTheme().default_size
        self.set_title(title)
        self.set_icon(image)
        self.set_size(size)
        self.extra(extra)  # noqa: S610
        self.border(border)

    def get_more_button(self):
        return self._extra_button

    def set_title(self, title: str = "") -> None:
        if title:
            self._title_label.setText(title)
            self._title_label.setVisible(True)
            self._title_icon.setVisible(self._title_icon_pixmap is not None)
            self._title_divider.setVisible(True)
            self._main_layout.removeItem(self._title_layout)
            self._main_layout.insertLayout(0, self._title_layout)
        else:
            self._title_label.setVisible(False)
            self._title_icon.setVisible(False)
            self._title_divider.setVisible(False)
            self._main_layout.removeItem(self._title_layout)

    def set_icon(self, icon: QtGui.QPixmap | None = None) -> None:
        if icon:
            self._title_icon.set_dayu_image(icon)
            self._title_icon.setVisible(True)
        else:
            self._title_icon.setVisible(False)

    def set_content(self, widget: QtWidgets.QWidget | None = None) -> None:
        if self._content_layout.count() > 0:
            if item := self._content_layout.itemAt(0).widget():
                item.deleteLater()

        if widget:
            self._content_layout.addWidget(widget)
        else:
            self._content_layout.addStretch()

    def set_size(self, size: int | None = None) -> None:
        if size:
            title_props_map: dict[int, MCardTitleProps] = {
                MTheme().large: MCardTitleProps(MLabel.LabelLevel.H2, 20),
                MTheme().medium: MCardTitleProps(MLabel.LabelLevel.H3, 15),
                MTheme().small: MCardTitleProps(MLabel.LabelLevel.H4, 10),
            }
            title_props = title_props_map.get(
                size, MCardTitleProps(MLabel.LabelLevel.H4, 10)
            )
            self._title_label.set_dayu_level(title_props.level)
            padding = title_props.padding
            self._title_layout.setContentsMargins(padding, padding, padding, padding)
            self._title_icon.set_dayu_size(size)

    def extra(self, show: bool = True) -> MCard:
        if show:
            self._extra_button.setVisible(True)
        else:
            self._extra_button.setVisible(False)
        return self

    def border(self, border: bool = True):
        self.setProperty("border", border)
        self.style().polish(self)
        return self

    def _on_extra_menu_clicked(self):
        context_menu = MMenu(parent=self)
        for name, callback in self._extra_menus:
            context_menu.addAction(name, callback)
        context_menu.exec(self.mapToGlobal(self._extra_button.geometry().bottomLeft()))


@hover_shadow_mixin
@cursor_mixin
class MMetaCard(QtWidgets.QWidget):
    def __init__(
        self,
        title: str = "",
        description: str = "",
        avatar: QtGui.QPixmap | None = None,
        cover: QtGui.QPixmap | None = None,
        extra: bool = False,
        extra_menus: list[tuple[str, Callable[[], None]]] | None = None,
        border: bool = True,
        parent: QtWidgets.QWidget | None = None,
    ):
        super().__init__(parent)
        self.setAttribute(QtCore.Qt.WidgetAttribute.WA_StyledBackground)
        self._title_label = MLabel().h4()
        self._description_label = MLabel().secondary()
        self._description_label.setWordWrap(True)
        self._description_label.set_elide_mode(QtCore.Qt.TextElideMode.ElideRight)
        self._avatar = MAvatar()
        self._cover_label = QtWidgets.QLabel()

        self._title_layout = QtWidgets.QHBoxLayout()
        self._title_layout.addWidget(self._title_label)
        self._title_layout.addStretch()
        self._extra_button = MToolButton(parent=self).icon_only().svg("more.svg")
        self._extra_menus = extra_menus or []
        self._extra_button.clicked.connect(self._on_extra_menu_clicked)
        self._title_layout.addWidget(self._extra_button)
        self._extra_button.setVisible(extra)

        content_layout = QtWidgets.QFormLayout()
        content_layout.setContentsMargins(5, 5, 5, 5)
        content_layout.addRow(self._avatar, self._title_layout)
        content_layout.addRow(self._description_label)

        self._button_layout = QtWidgets.QHBoxLayout()

        main_layout = QtWidgets.QVBoxLayout()
        main_layout.setSpacing(0)
        main_layout.setContentsMargins(1, 1, 1, 1)
        main_layout.addWidget(self._cover_label)
        main_layout.addLayout(content_layout)
        main_layout.addLayout(self._button_layout)
        main_layout.addStretch()
        self.setLayout(main_layout)
        self._cover_label.setFixedSize(QtCore.QSize(200, 200))

        self.set_title(title)
        self.set_description(description)
        self.set_avatar(avatar)
        self.set_cover(cover)
        self.border(border)

    def get_more_button(self):
        return self._extra_button

    def set_title(self, title: str = "") -> None:
        if title:
            self._title_label.setText(title)
            self._title_label.setVisible(True)
        else:
            self._title_label.setVisible(False)

    def set_description(self, description: str = "") -> None:
        if description:
            self._description_label.setText(description)
            self._description_label.setVisible(True)
        else:
            self._description_label.setVisible(False)

    def set_avatar(self, avatar: QtGui.QPixmap | None = None) -> None:
        if avatar:
            self._avatar.set_dayu_image(avatar)
            self._avatar.setVisible(True)
        else:
            self._avatar.setVisible(False)

    def set_cover(self, cover: QtGui.QPixmap | None = None) -> None:
        if cover:
            fixed_height = self._cover_label.width()
            self._cover_label.setPixmap(
                cover.scaledToWidth(
                    fixed_height, QtCore.Qt.TransformationMode.SmoothTransformation
                )
            )
            self._cover_label.setVisible(True)
        else:
            self._cover_label.setVisible(False)

    def border(self, border: bool = True) -> MMetaCard:
        self.setProperty("border", border)
        self.style().polish(self)
        return self

    def _on_extra_menu_clicked(self):
        context_menu = MMenu(parent=self)
        for name, callback in self._extra_menus:
            context_menu.addAction(name, callback)
        context_menu.exec(self.mapToGlobal(self._extra_button.geometry().bottomLeft()))



import functools

from qtpy import QtWidgets

from qtui.dayu.alert import MAlert
from qtui.dayu.button_group import MPushButtonGroup
from qtui.dayu.divider import MDivider
from qtui.dayu.field_mixin import MFieldMixin
from qtui.dayu.label import MLabel
from qtui.dayu.types import MButtonGroupData


class AlertExample(QtWidgets.QWidget, MFieldMixin):
    def __init__(self, parent: QtWidgets.QWidget | None = None):
        super().__init__(parent)
        self.setWindowTitle("Example for MAlert")
        main_lay = QtWidgets.QVBoxLayout()
        self.setLayout(main_lay)
        main_lay.addWidget(MDivider("different type"))
        main_lay.addWidget(MAlert(text="Information Message", parent=self).info())
        main_lay.addWidget(MAlert(text="Success Message", parent=self).success())
        main_lay.addWidget(MAlert(text="Warning Message", parent=self).warning())
        main_lay.addWidget(MAlert(text="Error Message", parent=self).error())

        closable_alert = MAlert("Some Message", parent=self).closable()

        main_lay.addWidget(MLabel("Different Alert Types"))
        main_lay.addWidget(MDivider("closable"))
        main_lay.addWidget(closable_alert)
        main_lay.addWidget(MDivider("data bind"))
        self.register_field("msg", "")
        self.register_field("msg_type", MAlert.AlertType.Info)

        data_bind_alert = MAlert(parent=self)
        data_bind_alert.set_closable(True)

        self.bind("msg", data_bind_alert, "dayu_text")
        self.bind("msg_type", data_bind_alert, "dayu_type")
        button_grp = MPushButtonGroup()
        button_grp.set_button_list(
            [
                MButtonGroupData(
                    text="error",
                    clicked=functools.partial(
                        self.slot_change_alert,
                        "password is wrong",
                        MAlert.AlertType.Error,
                    ),
                ),
                MButtonGroupData(
                    text="success",
                    clicked=functools.partial(
                        self.slot_change_alert,
                        "login success",
                        MAlert.AlertType.Success,
                    ),
                ),
                MButtonGroupData(
                    text="no more error",
                    clicked=functools.partial(
                        self.slot_change_alert,
                        "",
                        MAlert.AlertType.Info,
                    ),
                ),
            ]
        )
        main_lay.addWidget(button_grp)
        main_lay.addWidget(data_bind_alert)
        main_lay.addStretch()

    def slot_change_alert(self, alert_text: str, alert_type: MAlert.AlertType):
        self.set_field("msg_type", alert_type)
        self.set_field("msg", alert_text)


if __name__ == "__main__":
    from qtui.dayu.qt import application
    from qtui.dayu.theme import MTheme

    with application() as app:
        test = AlertExample()
        MTheme().apply(test)
        test.show()



from qtpy import QtWidgets

from qtui.dayu.card import MCard, MMetaCard
from qtui.dayu.divider import MDivider
from qtui.dayu.flow_layout import MFlowLayout
from qtui.dayu.label import MLabel
from qtui.dayu.qt import MPixmap
from qtui.dayu.theme import MTheme
from qtui.dayu.types import MCardData, MMetaCardData


class CardExample(QtWidgets.QWidget):
    def __init__(self, parent: QtWidgets.QWidget | None = None):
        super().__init__(parent)
        self.setWindowTitle("Examples for MCard")
        self._init_ui()

    def _init_ui(self):
        basic_card_lay = MFlowLayout()
        basic_card_lay.setSpacing(20)
        for setting in [
            MCardData(
                title="",
            ),
            MCardData(title="Card Title", size=MTheme().small),
            MCardData(title="Card Title", image=MPixmap("app-houdini.png")),
            MCardData(
                title="Card Title",
                extra=True,
                image=MPixmap("app-houdini.png"),
            ),
            MCardData(
                title="Card Title",
                extra=True,
                extra_menus=[("extra", lambda: print("extra"))],
            ),
        ]:
            card_0 = MCard(**setting)
            content_widget_0 = QtWidgets.QWidget()
            content_lay_0 = QtWidgets.QVBoxLayout()
            content_lay_0.setContentsMargins(15, 15, 15, 15)
            content_widget_0.setLayout(content_lay_0)
            for i in range(4):
                content_lay_0.addWidget(MLabel(f"Card Content {i + 1}"))
            card_0.set_content(content_widget_0)
            card_0.set_title("")
            card_0.set_title(setting.get("title", ""))

            basic_card_lay.addWidget(card_0)

        meta_card_lay = MFlowLayout()
        meta_card_lay.setSpacing(20)
        for setting in [
            MMetaCardData(
                title="Houdini",
                description=(
                    "Side Effects Software's flagship product, "
                    "an effective tool for creating advanced visual effects"
                ),
                avatar=MPixmap("user_line.svg"),
                cover=MPixmap("app-houdini.png"),
            ),
            MMetaCardData(
                title="Autodesk Maya",
                description=(
                    "The world's leading software application for 3D digital "
                    "animation and visual effects"
                ),
                cover=MPixmap("app-maya.png"),
            ),
        ]:
            meta_card = MMetaCard(**setting)
            meta_card_lay.addWidget(meta_card)

        task_card_lay = QtWidgets.QVBoxLayout()
        task_card_lay.setSpacing(10)
        for setting in [
            MMetaCardData(
                title="Task A",
                description="demo pl_0010 Animation \n2019/04/01 - 2019/04/09",
                avatar=MPixmap("success_line.svg", MTheme().success_color),
                extra=True,
                extra_menus=[("extra", lambda: print("extra"))],
            ),
            MMetaCardData(
                title="Task B",
                description="#2 closed by xiao hua.",
                avatar=MPixmap("error_line.svg", MTheme().error_color),
            ),
            MMetaCardData(
                title="Task C",
                description="#3 closed by xiao hua.",
                avatar=MPixmap("warning_line.svg", MTheme().warning_color),
            ),
        ] * 5:
            meta_card = MMetaCard(**setting)
            task_card_lay.addWidget(meta_card)

        left_lay = QtWidgets.QVBoxLayout()
        left_lay.addWidget(MDivider("Basic"))
        left_lay.addLayout(basic_card_lay)
        left_lay.addWidget(MDivider("Meta E-Commerce Example"))
        left_lay.addLayout(meta_card_lay)
        left_lay.addStretch()
        left_widget = QtWidgets.QWidget()
        left_widget.setLayout(left_lay)

        right_lay = QtWidgets.QVBoxLayout()
        right_lay.addWidget(MDivider("Meta Task Item Example"))
        scroll = QtWidgets.QScrollArea()
        scroll.setWidgetResizable(True)
        task_widget = QtWidgets.QWidget()
        task_widget.setLayout(task_card_lay)
        scroll.setWidget(task_widget)

        right_lay.addWidget(scroll)
        right_widget = QtWidgets.QWidget()
        right_widget.setLayout(right_lay)

        splitter = QtWidgets.QSplitter()
        splitter.addWidget(left_widget)
        splitter.addWidget(right_widget)
        splitter.setStretchFactor(0, 80)
        splitter.setStretchFactor(1, 20)
        main_lay = QtWidgets.QVBoxLayout()
        main_lay.addWidget(splitter)
        self.setLayout(main_lay)


if __name__ == "__main__":
    from qtui.dayu.qt import application
    from qtui.dayu.theme import MTheme

    with application() as app:
        test = CardExample()
        MTheme().apply(test)
        test.show()



import codecs
import importlib
import os
from pathlib import Path

from qtpy import QtCore, QtWidgets

from qtui.dayu.dock_widget import MDockWidget
from qtui.dayu.item_view_set import MItemViewSet
from qtui.dayu.theme import MTheme
from qtui.dayu.types import ModelData


def get_test_widget() -> list[tuple[str, type[QtWidgets.QWidget], list[str]]]:
    result: list[tuple[str, type[QtWidgets.QWidget], list[str]]] = []
    file = Path(__file__)
    directory = file.parent
    for f in directory.iterdir():
        if f.stem.startswith("__") or (not f.suffix == ".py") or f.name == file.name:
            continue
        name = f.stem
        module_name = f"examples.dayu.{name}"
        class_name = "".join(x.title() for x in name.split("_"))
        module = importlib.import_module(module_name, class_name)
        if hasattr(module, class_name):
            with codecs.open(
                os.path.join(directory, f"{name}.py"), encoding="utf-8"
            ) as f:
                result.append((name, getattr(module, class_name), f.readlines()))
    return result


class MDemo(QtWidgets.QMainWindow):
    def __init__(self, parent: QtWidgets.QWidget | None = None):
        super().__init__(parent)
        self.setWindowTitle("Dayu Widgets Demo")
        self._init_ui()

    def _init_ui(self):
        self.text_edit = QtWidgets.QTextEdit()
        self.stacked_widget = QtWidgets.QStackedWidget()

        list_widget = MItemViewSet(view_type=MItemViewSet.ListViewType)
        list_widget.set_header_list(
            [{"key": "name", "label": "Name", "icon": "list_view.svg"}]
        )
        list_widget.sig_left_clicked.connect(self.slot_change_widget)
        data_list: list[ModelData] = []
        for _index, (name, cls, code) in enumerate(get_test_widget()):
            data_list.append({"name": name[:-8], "data": code})
            if not callable(cls):
                continue
            widget = cls()
            widget.setProperty("code", code)
            self.stacked_widget.addWidget(widget)
        list_widget.setup_data(data_list)

        test_widget = MDockWidget("Example List")
        test_widget.setWidget(list_widget)
        self.addDockWidget(QtCore.Qt.DockWidgetArea.LeftDockWidgetArea, test_widget)

        code_widget = MDockWidget("Example Code")
        code_widget.setWidget(self.text_edit)
        self.addDockWidget(QtCore.Qt.DockWidgetArea.RightDockWidgetArea, code_widget)
        self.setCentralWidget(self.stacked_widget)

    def slot_change_widget(self, index: QtCore.QModelIndex):
        self.stacked_widget.setCurrentIndex(index.row())
        widget = self.stacked_widget.widget(index.row())
        self.text_edit.setPlainText("".join(widget.property("code")))


if __name__ == "__main__":
    import sys
    from pathlib import Path

    sys.path.append(Path(__file__).parents[2].as_posix())

    from qtui.dayu.qt import application
    from qtui.dayu.theme import MTheme

    with application() as app:
        test = MDemo()
        MTheme().apply(test)
        test.show()



# Import local modules
from qtui.dayu.theme import MTheme
from qtui.dayu.types import HeaderData, ModelData


def score_color(score: float, _: ModelData):
    if score < 60:
        return MTheme().error_color
    elif score < 80:
        return MTheme().warning_color
    elif score >= 90:
        return MTheme().success_color
    return MTheme().info_color


def font_underline(x: float, _: ModelData):
    return {"underline": True}


def font_bold(x: float, _: ModelData):
    return {"bold": True}


def sex_icon_color(x: str, _: ModelData):
    return (f"{x.lower()}.svg", "skyblue" if x == "Male" else "pink")


def age_display(x: float, _: ModelData):
    return f"{x} years old"


def city_display(x: list[str] | str, _: ModelData):
    if isinstance(x, list):
        return " & ".join(x)
    return x


def city_bg_color(x: str, _: ModelData):
    if x:
        return "transparent"
    return MTheme().error_color


header_list: list[HeaderData] = [
    {
        "label": "Name",
        "key": "name",
        "checkable": True,
        "searchable": True,
        "width": 200,
        "font": font_underline,
        "icon": "user_fill.svg",
    },
    {
        "label": "Sex",
        "key": "sex",
        "searchable": True,
        "selectable": True,
        "icon": sex_icon_color,
    },
    {
        "label": "Age",
        "key": "age",
        "width": 90,
        "searchable": True,
        "editable": True,
        "display": age_display,
        "font": font_bold,
    },
    {
        "label": "Address",
        "key": "city",
        "selectable": True,
        "searchable": True,
        "exclusive": False,
        "width": 120,
        "display": city_display,
        "bg_color": city_bg_color,
    },
    {
        "label": "Score",
        "key": "score",
        "searchable": True,
        "editable": True,
        "order": "desc",
        "bg_color": score_color,
        "color": "#fff",
    },
    {"label": "Score Copy", "key": "score", "searchable": True, "color": score_color},
]

data_list: list[ModelData] = [
    {
        "name": "John Brown",
        "sex": "Male",
        "sex_list": ["Male", "Female"],
        "age": 18,
        "score": 89,
        "city": "New York",
        "city_list": ["New York", "Ottawa", "London", "Sydney"],
        "date": "2016-10-03",
    },
    {
        "name": "Jim Green",
        "sex": "Male",
        "sex_list": ["Male", "Female"],
        "age": 24,
        "score": 55,
        "city": "London",
        "city_list": ["New York", "Ottawa", "London", "Sydney"],
        "date": "2016-10-01",
    },
    {
        "name": "Zhang Xiaoming",
        "sex": "Male",
        "sex_list": ["Male", "Female"],
        "age": 30,
        "score": 70,
        "city": "",
        "city_list": ["Beijing", "Shanghai", "Shenzhen", "Guangzhou"],
        "date": "2016-10-02",
    },
    {
        "name": "Jon Snow",
        "sex": "Female",
        "sex_list": ["Male", "Female"],
        "age": 26,
        "score": 60,
        "city": "Ottawa",
        "city_list": ["New York", "Ottawa", "London", "Sydney"],
        "date": "2016-10-04",
    },
    {
        "name": "Li Xiaohua",
        "sex": "Female",
        "sex_list": ["Male", "Female"],
        "age": 18,
        "score": 97,
        "city": "Ottawa",
        "city_list": ["New York", "Ottawa", "London", "Sydney"],
        "date": "2016-10-04",
    },
]

tree_data_list: list[ModelData] = [
    {
        "name": "John Brown",
        "sex": "Male",
        "sex_list": ["Male", "Female"],
        "age": 18,
        "score": 89,
        "city": "New York",
        "city_list": ["New York", "Ottawa", "London", "Sydney"],
        "date": "2016-10-03",
        "children": [
            {
                "name": "Jim Green",
                "sex": "Male",
                "sex_list": ["Male", "Female"],
                "age": 24,
                "score": 55,
                "city": "London",
                "city_list": ["New York", "Ottawa", "London", "Sydney"],
                "date": "2016-10-01",
            },
            {
                "name": "Zhang Xiaoming",
                "sex": "Male",
                "sex_list": ["Male", "Female"],
                "age": 30,
                "score": 70,
                "city": "",
                "city_list": ["Beijing", "Shanghai", "Shenzhen", "Guangzhou"],
                "date": "2016-10-02",
            },
        ],
    },
    {
        "name": "Jon Snow",
        "sex": "Female",
        "sex_list": ["Male", "Female"],
        "age": 26,
        "score": 60,
        "city": "Ottawa",
        "city_list": ["New York", "Ottawa", "London", "Sydney"],
        "date": "2016-10-04",
    },
    {
        "name": "Li Xiaohua",
        "sex": "Female",
        "sex_list": ["Male", "Female"],
        "age": 18,
        "score": 97,
        "city": "Ottawa",
        "city_list": ["New York", "Ottawa", "London", "Sydney"],
        "date": "2016-10-04",
    },
]



from qtpy import QtWidgets

import examples.dayu._mock_data as mock
from qtui.dayu.field_mixin import MFieldMixin
from qtui.dayu.item_model import MSortFilterModel, MTableModel
from qtui.dayu.item_view import MTreeView
from qtui.dayu.line_edit import MLineEdit
from qtui.dayu.push_button import MPushButton
from qtui.dayu.theme import MTheme


class TreeViewExample(QtWidgets.QWidget, MFieldMixin):
    def __init__(self, parent: QtWidgets.QWidget | None = None):
        super().__init__(parent)
        self._init_ui()

    def _init_ui(self):
        model_1 = MTableModel()
        model_1.set_header_list(mock.header_list)
        model_sort = MSortFilterModel()
        model_sort.setSourceModel(model_1)

        tree_view = MTreeView()
        tree_view.setModel(model_sort)

        model_sort.set_header_list(mock.header_list)
        tree_view.set_header_list(mock.header_list)
        model_1.set_data_list(mock.tree_data_list)

        line_edit = MLineEdit().search_with_close().small()
        line_edit.textChanged.connect(model_sort.set_search_pattern)

        expand_all_button = MPushButton("Expand All").small()
        expand_all_button.clicked.connect(tree_view.expandAll)
        collapse_all_button = MPushButton("Collapse All").small()
        collapse_all_button.clicked.connect(tree_view.collapseAll)
        button_lay = QtWidgets.QHBoxLayout()
        button_lay.addWidget(expand_all_button)
        button_lay.addWidget(collapse_all_button)
        button_lay.addWidget(line_edit)
        button_lay.addStretch()

        main_lay = QtWidgets.QVBoxLayout()
        main_lay.addLayout(button_lay)
        main_lay.addWidget(tree_view)
        main_lay.addStretch()
        self.setLayout(main_lay)


if __name__ == "__main__":
    from qtui.dayu.qt import application
    from qtui.dayu.theme import MTheme

    with application() as app:
        test = TreeViewExample()
        MTheme().apply(test)
        test.show()



"""
MToast
"""

from typing import ClassVar

from qtpy import QtCore, QtGui, QtWidgets

from .avatar import MAvatar
from .label import MLabel
from .loading import MLoading
from .qt import MPixmap
from .theme import MTheme


class MToast(QtWidgets.QWidget):
    """
    MToast
    A Phone style message.
    """

    InfoType = "info"
    SuccessType = "success"
    WarningType = "warning"
    ErrorType = "error"
    LoadingType = "loading"

    default_config: ClassVar[dict[str, int]] = {
        "duration": 2,
    }

    sig_closed = QtCore.Signal()

    def __init__(
        self,
        text: str,
        duration: int | None = None,
        dayu_type: str | None = None,
        parent: QtWidgets.QWidget | None = None,
    ):
        super().__init__(parent)
        self.setWindowFlags(
            QtCore.Qt.WindowType.FramelessWindowHint | QtCore.Qt.WindowType.Dialog
        )
        # self.setAttribute(QtCore.Qt.WidgetAttribute.WA_NoSystemBackground)
        self.setAttribute(QtCore.Qt.WidgetAttribute.WA_DeleteOnClose)
        self.setAttribute(QtCore.Qt.WidgetAttribute.WA_StyledBackground)

        _icon_lay = QtWidgets.QHBoxLayout()
        _icon_lay.addStretch()

        if dayu_type == MToast.LoadingType:
            _icon_lay.addWidget(
                MLoading(size=MTheme().huge, color=MTheme().text_color_inverse)
            )
        else:
            _icon_label = MAvatar()
            _icon_label.set_dayu_size(MTheme().toast_icon_size)
            _icon_label.set_dayu_image(
                MPixmap(
                    f"{dayu_type or MToast.InfoType}_line.svg",
                    MTheme().text_color_inverse,
                )
            )
            _icon_lay.addWidget(_icon_label)
        _icon_lay.addStretch()

        _content_label = MLabel()
        _content_label.setText(text)
        _content_label.setAlignment(QtCore.Qt.AlignmentFlag.AlignCenter)

        _main_lay = QtWidgets.QVBoxLayout()
        _main_lay.setContentsMargins(0, 0, 0, 0)
        _main_lay.addStretch()
        _main_lay.addLayout(_icon_lay)
        _main_lay.addSpacing(10)
        _main_lay.addWidget(_content_label)
        _main_lay.addStretch()
        self.setLayout(_main_lay)
        self.setFixedSize(QtCore.QSize(MTheme().toast_size, MTheme().toast_size))

        _close_timer = QtCore.QTimer(self)
        _close_timer.setSingleShot(True)
        _close_timer.timeout.connect(self.close)
        _close_timer.timeout.connect(self.sig_closed)
        _close_timer.setInterval((duration or self.default_config["duration"]) * 1000)
        self.has_played = False

        if dayu_type != MToast.LoadingType:
            _close_timer.start()

        self._opacity_ani = QtCore.QPropertyAnimation()
        self._opacity_ani.setTargetObject(self)
        self._opacity_ani.setDuration(300)
        self._opacity_ani.setEasingCurve(QtCore.QEasingCurve.Type.OutCubic)
        self._opacity_ani.setPropertyName(b"windowOpacity")
        self._opacity_ani.setStartValue(0.0)
        self._opacity_ani.setEndValue(0.9)

        self._get_center_position(parent)
        self._fade_int()

    def closeEvent(self, event: QtGui.QCloseEvent):  # noqa: N802
        if self.has_played:
            event.accept()
        else:
            self._fade_out()
            event.ignore()

    def _fade_out(self) -> None:
        self.has_played = True
        self._opacity_ani.setDirection(QtCore.QAbstractAnimation.Direction.Backward)
        self._opacity_ani.finished.connect(self.close)
        self._opacity_ani.start()

    def _fade_int(self) -> None:
        self._opacity_ani.start()

    def _get_center_position(self, parent: QtWidgets.QWidget | None) -> None:
        if parent is None:
            return
        parent_geo = parent.geometry()
        pos = parent.mapToGlobal(parent_geo.topLeft())
        offset = 0
        for child in parent.children():
            if isinstance(child, MToast) and child.isVisible():
                offset = max(offset, child.y())
        target_x = pos.x() + parent_geo.width() / 2 - self.width() / 2
        target_y = pos.y() + parent_geo.height() / 2 - self.height() / 2
        self.setProperty("pos", QtCore.QPoint(int(target_x), int(target_y)))

    @classmethod
    def info(cls, text: str, parent: QtWidgets.QWidget, duration: int | None = None):
        """Show a normal toast message"""
        inst = cls(text, duration=duration, dayu_type=MToast.InfoType, parent=parent)
        inst.show()
        return inst

    @classmethod
    def success(cls, text: str, parent: QtWidgets.QWidget, duration: int | None = None):
        """Show a success toast message"""
        inst = cls(text, duration=duration, dayu_type=MToast.SuccessType, parent=parent)
        inst.show()
        return inst

    @classmethod
    def warning(cls, text: str, parent: QtWidgets.QWidget, duration: int | None = None):
        """Show a warning toast message"""
        inst = cls(text, duration=duration, dayu_type=MToast.WarningType, parent=parent)
        inst.show()
        return inst

    @classmethod
    def error(cls, text: str, parent: QtWidgets.QWidget, duration: int | None = None):
        """Show an error toast message"""
        inst = cls(text, duration=duration, dayu_type=MToast.ErrorType, parent=parent)
        inst.show()
        return inst

    @classmethod
    def loading(cls, text: str, parent: QtWidgets.QWidget):
        """Show a toast message with loading animation.
        You should close this widget by yourself."""
        inst = cls(text, dayu_type=MToast.LoadingType, parent=parent)
        inst.show()
        return inst

    @classmethod
    def config(cls, duration: int | None = None):
        """
        Config the global MToast duration setting.
        :param duration: int (unit is second)
        :return: None
        """
        if duration is not None:
            cls.default_config["duration"] = duration



"""
MCollapse
"""

from __future__ import annotations

from qtpy import QtCore, QtWidgets

from .label import MLabel
from .mixin import property_mixin
from .qt import MPixmap
from .tool_button import MToolButton


@property_mixin
class MCollapse(QtWidgets.QWidget):
    sig_context_menu = QtCore.Signal(object)

    def __init__(
        self,
        title: str = "",
        content: QtWidgets.QWidget | None = None,
        expand: bool = False,
        closable: bool = False,
        parent: QtWidgets.QWidget | None = None,
    ):
        super().__init__(parent)
        self.setAttribute(QtCore.Qt.WidgetAttribute.WA_StyledBackground)

        self.title_label = MLabel(parent=self)
        self.expand_icon = MLabel(parent=self)
        self.expand_icon.setSizePolicy(
            QtWidgets.QSizePolicy.Policy.Minimum, QtWidgets.QSizePolicy.Policy.Minimum
        )
        self._close_button = MToolButton().icon_only().tiny().svg("close_line.svg")
        self._close_button.clicked.connect(self.close)
        self._closable = closable
        self._expand = expand
        self._content = content

        self._central_widget = None

        header_layout = QtWidgets.QHBoxLayout()
        header_layout.addWidget(self.expand_icon)
        header_layout.addWidget(self.title_label)
        header_layout.addStretch()
        header_layout.addWidget(self._close_button)

        self.header_widget = QtWidgets.QWidget(parent=self)
        self.header_widget.setAttribute(QtCore.Qt.WidgetAttribute.WA_StyledBackground)
        self.header_widget.setObjectName("title")
        self.header_widget.setLayout(header_layout)
        self.header_widget.setSizePolicy(
            QtWidgets.QSizePolicy.Policy.Minimum, QtWidgets.QSizePolicy.Policy.Minimum
        )
        self.header_widget.setCursor(QtCore.Qt.CursorShape.PointingHandCursor)
        self.title_label.setCursor(QtCore.Qt.CursorShape.PointingHandCursor)
        self.header_widget.installEventFilter(self)
        self.title_label.installEventFilter(self)

        self.content_widget = QtWidgets.QWidget(parent=self)
        self.content_layout = QtWidgets.QHBoxLayout()
        self.content_widget.setLayout(self.content_layout)

        self.main_layout = QtWidgets.QVBoxLayout()
        self.main_layout.setContentsMargins(0, 0, 0, 0)
        self.main_layout.setSpacing(0)
        self.main_layout.addWidget(self.header_widget)
        self.main_layout.addWidget(self.content_widget)
        self.setLayout(self.main_layout)
        self.setSizePolicy(
            QtWidgets.QSizePolicy.Policy.Expanding, QtWidgets.QSizePolicy.Policy.Minimum
        )
        self.setMouseTracking(True)

        self.set_title(title)
        self.set_content(content)
        self.closable(closable)
        self.expand(expand)

    def set_title(self, value: str) -> None:
        self.title_label.setText(value)

    def set_content(self, widget: QtWidgets.QWidget | None) -> None:
        if widget is not None:
            if self._content:
                self.content_layout.removeWidget(self._content)
                self._content.close()
            self.content_layout.addWidget(widget)
            self._content = widget

    def content(self) -> QtWidgets.QWidget | None:
        return self._content

    def closable(self, value: bool = True) -> MCollapse:
        self._closable = value
        self.content_widget.setVisible(value)
        self._close_button.setVisible(value)
        return self

    def expand(self, value: bool = True) -> MCollapse:
        self._expand = value
        self.content_widget.setVisible(value)
        self.expand_icon.setPixmap(
            MPixmap("down_line.svg" if value else "right_line.svg").scaledToHeight(12)
        )
        return self

    def eventFilter(self, widget: QtWidgets.QWidget, event: QtCore.QEvent) -> bool:  # noqa: N802  # pyright: ignore[reportIncompatibleMethodOverride]
        if (
            widget in [self.header_widget, self.title_label]
            and event.type() == QtCore.QEvent.Type.MouseButtonRelease
        ):
            self.expand(not self._expand)
        return super().eventFilter(widget, event)



"""MStackedWidget"""

# Import third-party modules
from qtpy import QtWidgets

# Import local modules
from .mixin import stacked_animation_mixin


@stacked_animation_mixin
class MStackedWidget(QtWidgets.QStackedWidget):
    """Just active animation when current index changed."""

    def __init__(self, parent: QtWidgets.QWidget | None = None):
        super().__init__(parent)

    def disable_animation(self):
        self.currentChanged.disconnect(self._play_anim)  # pyright: ignore[reportUnknownMemberType, reportUnknownArgumentType, reportAttributeAccessIssue]



from qtpy import QtCore, QtWidgets

from qtui.dayu.button_group import MCheckBoxGroup
from qtui.dayu.divider import MDivider
from qtui.dayu.field_mixin import MFieldMixin
from qtui.dayu.label import MLabel
from qtui.dayu.push_button import MPushButton
from qtui.dayu.qt import MIcon
from qtui.dayu.types import MButtonGroupData


class CheckBoxGroupExample(QtWidgets.QWidget, MFieldMixin):
    def __init__(self, parent: QtWidgets.QWidget | None = None):
        super().__init__(parent)
        self._init_ui()

    def _init_ui(self):
        self.app_data = [
            MButtonGroupData(text="Maya", icon=MIcon("app-maya.png")),
            MButtonGroupData(text="Nuke", icon=MIcon("app-nuke.png")),
            MButtonGroupData(text="Houdini", icon=MIcon("app-houdini.png")),
        ]
        radio_group_h = MCheckBoxGroup()
        radio_group_v = MCheckBoxGroup(orientation=QtCore.Qt.Orientation.Vertical)

        radio_group_h.set_button_list(self.app_data)
        radio_group_v.set_button_list(self.app_data)

        self.data_list = [
            "Beijing",
            "Shanghai",
            "Guangzhou",
            "Shenzhen",
            "Zhengzhou",
            "Shijiazhuang",
        ]
        radio_group_b = MCheckBoxGroup()
        radio_group_b.set_button_list(self.data_list)

        button = MPushButton(text="Change Value")
        button.clicked.connect(self.slot_button_clicked)

        label = MLabel()
        self.register_field("checked_app", ["Beijing", "Shanghai"])
        self.register_field(
            "checked_app_text", lambda: " & ".join(self.field("checked_app"))
        )
        self.bind(
            "checked_app", radio_group_b, "dayu_checked", signal="checked_changed"
        )
        self.bind("checked_app_text", label, "text")

        radio_group_tri = MCheckBoxGroup()
        radio_group_tri.set_context_menu_properties(
            show=True,
            select_all="select all",
            select_none="select none",
            select_invert="select invert",
        )
        radio_group_tri.set_button_list(self.app_data)
        self.register_field("check_grp", ["Maya"])
        self.bind(
            "check_grp", radio_group_tri, "dayu_checked", signal="checked_changed"
        )

        main_lay = QtWidgets.QVBoxLayout()
        main_lay.addWidget(MDivider("Orientation Qt.Horizontal"))
        main_lay.addWidget(radio_group_h)
        main_lay.addWidget(MDivider("Orientation Qt.Vertical"))
        main_lay.addWidget(radio_group_v)

        main_lay.addWidget(MDivider("Data Bind"))
        main_lay.addWidget(radio_group_b)
        main_lay.addWidget(label)
        main_lay.addWidget(button)

        main_lay.addWidget(MDivider("Try Context Menu"))
        main_lay.addWidget(radio_group_tri)
        main_lay.addStretch()
        self.setLayout(main_lay)

    @QtCore.Slot()
    def slot_button_clicked(self):
        import random

        start = random.randint(0, len(self.data_list))
        end = random.randint(start, len(self.data_list))
        self.set_field("checked_app", self.data_list[start:end])


if __name__ == "__main__":
    from qtui.dayu.qt import application
    from qtui.dayu.theme import MTheme

    with application() as app:
        test = CheckBoxGroupExample()
        MTheme().apply(test)
        test.show()



"""
MAvatar.
"""

from __future__ import annotations

from qtpy import QtCore, QtGui, QtWidgets

from .qt import MPixmap
from .theme import MTheme
from .widget import MWidget


class MAvatar(QtWidgets.QLabel, MWidget):
    """
    Avatar component. It can be used to represent people or object.
    Property:
        image: avatar image, should be QPixmap.
        dayu_size: the size of image.
    """

    def __init__(
        self,
        parent: QtWidgets.QWidget | None = None,
        flags: QtCore.Qt.WindowType = QtCore.Qt.WindowType.Widget,
    ):
        super().__init__(parent, flags)
        self._default_pix = MPixmap("user_fill.svg")
        self._pixmap = self._default_pix
        self._dayu_size = 0
        self.set_dayu_size(MTheme().default_size)

    def set_dayu_size(self, value: int) -> None:
        """
        Set the avatar size.
        :param value: integer
        :return: None
        """
        if value != self._dayu_size:
            self._dayu_size = value
            self._set_size()

    def _set_size(self) -> None:
        self.setFixedSize(QtCore.QSize(self._dayu_size, self._dayu_size))
        self._set_image()

    def _set_image(self) -> None:
        self.setPixmap(
            self._pixmap.scaledToWidth(
                self.height(), QtCore.Qt.TransformationMode.SmoothTransformation
            )
        )

    def set_dayu_image(self, value: QtGui.QPixmap | None):
        """
        Set avatar image.
        :param value: QPixmap or None.
        :return: None
        """

        if value is None:
            self._pixmap = self._default_pix
        else:
            self._pixmap = self._default_pix if value.isNull() else value

        self._set_image()

    def get_dayu_image(self) -> QtGui.QPixmap:
        """
        Get the avatar image.
        :return: QPixmap
        """
        return self._pixmap

    def get_dayu_size(self) -> int:
        """
        Get the avatar size
        :return: integer
        """
        return self._dayu_size

    dayu_image = QtCore.Property(QtGui.QPixmap, get_dayu_image, set_dayu_image)
    dayu_size = QtCore.Property(int, get_dayu_size, set_dayu_size)

    @classmethod
    def huge(cls, image: QtGui.QPixmap | None = None) -> MAvatar:
        """Create a MAvatar with huge size"""
        inst = cls()
        inst.set_dayu_size(MTheme().huge)
        inst.set_dayu_image(image)
        return inst

    @classmethod
    def large(cls, image: QtGui.QPixmap | None = None) -> MAvatar:
        """Create a MAvatar with large size"""
        inst = cls()
        inst.set_dayu_size(MTheme().large)
        inst.set_dayu_image(image)
        return inst

    @classmethod
    def medium(cls, image: QtGui.QPixmap | None = None) -> MAvatar:
        """Create a MAvatar with medium size"""
        inst = cls()
        inst.set_dayu_size(MTheme().medium)
        inst.set_dayu_image(image)
        return inst

    @classmethod
    def small(cls, image: QtGui.QPixmap | None = None) -> MAvatar:
        """Create a MAvatar with small size"""
        inst = cls()
        inst.set_dayu_size(MTheme().small)
        inst.set_dayu_image(image)
        return inst

    @classmethod
    def tiny(cls, image: QtGui.QPixmap | None = None) -> MAvatar:
        """Create a MAvatar with tiny size"""
        inst = cls()
        inst.set_dayu_size(MTheme().tiny)
        inst.set_dayu_image(image)
        return inst



import functools

from qtpy import QtCore, QtWidgets

from qtui.dayu.button_group import MPushButtonGroup
from qtui.dayu.divider import MDivider
from qtui.dayu.field_mixin import MFieldMixin
from qtui.dayu.slider import MSlider
from qtui.dayu.spin_box import MSpinBox
from qtui.dayu.types import MButtonGroupData


class SliderExample(QtWidgets.QWidget, MFieldMixin):
    def __init__(self, parent: QtWidgets.QWidget | None = None):
        super().__init__(parent)
        self.setWindowTitle("Examples for MSlider")
        self._init_ui()

    def _init_ui(self):
        self.register_field("percent", 20)
        main_lay = QtWidgets.QVBoxLayout()
        main_lay.addWidget(MDivider("different orientation"))
        for orn in [QtCore.Qt.Orientation.Horizontal, QtCore.Qt.Orientation.Vertical]:
            line_edit_hor = MSlider(orn)
            line_edit_hor.setRange(1, 100)
            self.bind("percent", line_edit_hor, "value")
            lay = QtWidgets.QVBoxLayout()
            lay.addWidget(line_edit_hor)
            main_lay.addLayout(lay)
        spin_box = MSpinBox()
        spin_box.setRange(1, 100)
        self.bind("percent", spin_box, "value", signal="valueChanged")

        lay3 = QtWidgets.QHBoxLayout()
        button_grp = MPushButtonGroup()
        button_grp.set_button_list(
            [
                MButtonGroupData(
                    text="+", clicked=functools.partial(self.slot_change_value, 10)
                ),
                MButtonGroupData(
                    text="-", clicked=functools.partial(self.slot_change_value, -10)
                ),
            ]
        )
        lay3.addWidget(spin_box)
        lay3.addWidget(button_grp)
        lay3.addStretch()
        main_lay.addWidget(MDivider("data bind"))
        main_lay.addLayout(lay3)
        main_lay.addStretch()
        self.setLayout(main_lay)

    def slot_change_value(self, value: int):
        self.set_field("percent", max(0, min(self.field("percent") + value, 100)))


if __name__ == "__main__":
    from qtui.dayu.qt import application
    from qtui.dayu.theme import MTheme

    with application() as app:
        test = SliderExample()
        MTheme().apply(test)
        test.show()



import functools
from collections.abc import Callable
from typing import Any

from qtpy import QtWidgets

from .types import MixinComputed, MixinData, MixinProps
from .utils import qbytearray_to_str


class MFieldMixin:
    def __init__(self):
        self.computed_dict: dict[str, MixinComputed] = {}
        self.props_dict: dict[str, MixinProps] = {}

    def register_field(
        self,
        name: str,
        getter: Callable[[], Any] | Any = None,
        setter: Callable[[Any], None] | None = None,
        required: bool = False,
    ):
        if callable(getter):
            value = getter()
            self.computed_dict[name] = {
                "value": value,
                "getter": getter,
                "setter": setter,
                "required": required,
                "bind": [],
            }
        else:
            self.props_dict[name] = {"value": getter, "required": required, "bind": []}
        return

    def bind(
        self,
        data_name: str,
        widget: QtWidgets.QWidget,
        qt_property: str,
        index: int | None = None,
        signal: str | None = None,
        callback: Callable[[], Any] | None = None,
    ):
        # data_dict = {
        #     "data_name": data_name,
        #     "widget": widget,
        #     "widget_property": qt_property,
        #     "index": index,
        #     "callback": callback,
        # }
        data_dict = MixinData(
            data_name=data_name,
            widget=widget,
            widget_property=qt_property,
            index=index,
            callback=callback,
        )
        if data_name in self.computed_dict:
            self.computed_dict[data_name]["bind"].append(data_dict)
        else:
            self.props_dict[data_name]["bind"].append(data_dict)
        if signal:  # 用户操作绑定数据
            getattr(widget, signal).connect(
                functools.partial(self._slot_changed_from_user, data_dict)
            )
        self._data_update_ui(data_dict)
        return widget

    def fields(self):
        return self.props_dict.keys() | self.computed_dict.keys()

    def field(self, name: str) -> Any:
        if name in self.props_dict:
            return self.props_dict[name]["value"]
        elif name in self.computed_dict:
            computed = self.computed_dict[name]
            getter = computed["getter"]
            if getter:
                new_value = getter()
                self.computed_dict[name]["value"] = new_value
                return new_value
            return computed["value"]
        else:
            raise KeyError(f'There is no field named "{name}"')

    def set_field(self, name: str, value: Any):
        if name in self.props_dict:
            self.props_dict[name]["value"] = value
            self._slot_prop_changed(name)

        elif name in self.computed_dict:
            self.computed_dict[name]["value"] = value
            self._slot_prop_changed(name)

    def _data_update_ui(self, data_dict: MixinData):
        data_name = data_dict.get("data_name")
        widget = data_dict["widget"]
        index = data_dict["index"]
        widget_property = data_dict["widget_property"]
        callback = data_dict["callback"]
        value = None
        if index is None:
            value = self.field(data_name)
        elif isinstance(self.field(data_name), dict):
            value = self.field(data_name).get(index)
        elif isinstance(self.field(data_name), list):
            value = (
                self.field(data_name)[index]
                if index < len(self.field(data_name))
                else None
            )
        if widget.metaObject().indexOfProperty(
            widget_property
        ) > -1 or widget_property in [
            qbytearray_to_str(b) for b in widget.dynamicPropertyNames()
        ]:
            widget.setProperty(widget_property, value)
        else:
            if hasattr(widget, "set_field"):
                widget.set_field(widget_property, value)  # pyright: ignore[reportUnknownMemberType, reportAttributeAccessIssue]
        if callable(callback):
            callback()

    def _slot_prop_changed(self, property_name: str):
        for key, setting_dict in self.props_dict.items():
            if key == property_name:
                for data_dict in setting_dict["bind"]:
                    self._data_update_ui(data_dict)

        for setting_dict in self.computed_dict.values():
            for data_dict in setting_dict["bind"]:
                self._data_update_ui(data_dict)

    def _slot_changed_from_user(self, data_dict: MixinData, ui_value: Any):
        self._ui_update_data(data_dict, ui_value)

    def _ui_update_data(self, data_dict: MixinData, ui_value: Any):
        data_name = data_dict.get("data_name")
        index = data_dict.get("index", None)
        if index is None:
            self.set_field(data_name, ui_value)
        else:
            old_value = self.field(data_name)
            old_value[index] = ui_value
            self.set_field(data_name, old_value)
        if data_name in self.props_dict.keys():
            self._slot_prop_changed(data_name)

    def _is_complete(self):
        for name, data_dict in self.computed_dict.items() | self.props_dict.items():
            if data_dict["required"] and not self.field(name):
                return False
        return True
